/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.Basis;

import SudokuPIPkg.DataForms.DCoord;
import java.util.ArrayList;
import java.util.Collections;

public class Basics {

  /**
   * Karakter eltávolítása a szövegből.
   *
   * @param sText szöveg, ahonnan el kell távolítani a karakter(eke)t.
   * @param sChar eltávolítandó karakter.
   * @return az új szöveg
   */
  public static String deleteChar(String sText, String sChar) {
    String aChar, rString = "";
    final int len = sText.length();
    for (int c = 0; c < len; c++) {
      aChar = sText.substring(c, c + 1);
      if (!aChar.equals(sChar)) {
        rString += aChar;
      }
    }
    return (rString);
  }

  /**
   * Visszaadja, hogy a pozíciót hanyadik blokk tartalmazza.
   *
   * @param coord
   * @return blokk sorszáma
   */
  public static byte getBlocknumberFromPosition(DCoord coord) {
    return (byte) (((coord.getRow() / 3) * 3) + (coord.getCol() / 3));
  }

  /**
   * Visszaadja a pozíció blokkon belüli eltolódás-értékét.
   *
   * @param coord
   * @return blokkon belüli pozíció
   */
  public static byte getOffsetFromPosition(DCoord coord) {
    return (byte) (((coord.getRow() % 3) * 3) + (coord.getCol() % 3));
  }

  /**
   * Összehasonlít 2 koordinátát,
   *
   * @param coord1 mit
   * @param coord2 mivel
   * @return visszaadott érték bitjei: 0 - ha nincs egy munkatéren = 0 1 - ha
   * egy sorban vannak = 1 2 - ha egy oszlopban vannak = 2 3 - ha egy blokkban
   * vannak = 4 4 - ha a két pozíció azonos = 8 A kapcsolattípus alapján
   * visszaadott érvényes értékeket összedjuk, akkor megkapjuk a kapcsolattípust
   * a két cella között.
   */
  public static byte getWhichContact(DCoord coord1, DCoord coord2) {
    boolean cond;
    byte res = 0;
    cond = (coord1.getRow() == coord2.getRow());
    if (cond && (coord1.getCol() == coord2.getCol())) {
      res = 8;
    }
    cond = (coord1.getRow() / 3 == coord2.getRow() / 3);
    if (cond && (coord1.getCol() / 3 == coord2.getCol() / 3)) {
      res += 4;
    }
    if (coord1.getCol() == coord2.getCol()) {
      res += 2;
    }
    if (coord1.getRow() == coord2.getRow()) {
      res += 1;
    }
    return res;
  }

  /**
   * Visszaadja a blokkpozíció alapján a koordinátát.
   *
   * @param blockNumber hanyadik blokk
   * @param offset a blokkon belüli pozíció
   * @return 
   */
  public static DCoord getCoordFromBlockPosition(byte blockNumber, byte offset) {
    byte row = (byte) ((byte) ((blockNumber / 3) * 3) + (offset / 3));
    byte col = (byte) (((blockNumber % 3) * 3) + (offset % 3));
    return new DCoord(row, col);
  }

  /**
   * Karakter beillesztése a szövegbe, és a szöveg rendezése.
   *
   * @param sWord szöveg
   * @param sChar karakter
   * @return
   */
  public static String insertChar(String sWord, String sChar) {
    return sortStringChars(sWord + sChar);
  }

  /**
   * Visszaadja, hogy az adott pozíció beleesik-e az adott sorszámú blokkba.
   *
   * @param coord
   * @param blockNumber blokk-sorszám
   * @return
   */
  public static boolean isPositionInBlock(DCoord coord, byte blockNumber) {
    return getBlocknumberFromPosition(coord) == blockNumber;
  }

  /**
   * Egy 2 hosszúságú stringből kiválasztja azt az értéket, ami nem a number
   *
   * @param numbers a két szám
   * @param number az a szám, amitől eltérőt kell választani.
   * @return
   */
  public static byte otherNumber(String numbers, byte number) {
    boolean isFirst = Integer.parseInt(numbers.substring(0, 1)) == number;
    return Byte.parseByte(isFirst ? numbers.substring(1, 2) : numbers.substring(0, 1));
  }

  /**
   * Sorba rakja a kapott szöveg karaktereit a kódjuk alapján
   *
   * @param sText szöveg, amiben vannak az adott karakterek.
   * @return rendezett szöveg
   */
  public static String sortStringChars(String sText) {
    ArrayList<Character> charsList= new ArrayList<>();
    for (int i = 0; i < sText.length(); i++) {
      charsList.add(sText.charAt(i));
    }
    Collections.sort(charsList);
    String s="";
    for (Character char1 : charsList) {
      s+=char1;
    }
    return s;
  }

  /**
   * sztringek karaktereinek unióját képezi.
   *
   * @param str szövegek halmaza
   * @return
   */
  public static String unionSets(String... str) {
    String union = "";
    String sor;
    for (String sorok : str) {
      sor = sorok;
      for (int c = 0; c < sor.length(); c++) {
        if (!union.contains(sor.substring(c, c + 1))) {
          union += sor.substring(c, c + 1);
        }
      }
    }
    return sortStringChars(union);
  }

  /**
   * Halmazelméleti kivonás
   *
   * @param from Ebből a szövegből törlök.
   * @param what Ezeket a karaktereket.
   * @return
   */
  public static String subtractionSets(String from, String what) {
    String result = from, str1, str2;
    int pos;
    for (int c = 0; c < what.length(); c++) {
      while ((pos = result.indexOf(what.substring(c, c + 1))) >= 0) {
        str1 = (pos > 0 ? result.substring(0, pos - 1) : "");
        str2 = (pos < result.length() ? result.substring(pos + 1) : "");
        result = str1 + str2;
      }
    }
    return result;
  }

  /**
   * Az adott szöveget balra igazítja úgy, hogy kiegészíti az extendChar paraméterben
   * megadott karakterrel, ha pedig hosszabb, akkor a végéről levágja a megadott hosszra.
   * Az extendCharban hiába adunk meg több karaktert, úgyis csak az első karakterrel foglalkozik.
   * 
   * @param what a kiegészítendő szöveg.
   * @param extendChar a kiegészítő karakter.
   * @param length az eredményül várt szöveg hossza.
   * @return 
   */
  public static String padL(String what, String extendChar, int length) {
    String longString = what + repeat(extendChar.substring(0, 1), length);
    return longString.substring(0, length);
  }

  /**
   * Az adott szöveget jobbra igazítja úgy, hogy kiegészíti az extendChar paraméterben
   * megadott karakterrel, ha pedig hosszabb, akkor az elejéről levágja a megadott hosszra.
   * Az extendCharban hiába adunk meg több karaktert, úgyis csak az első karakterrel foglalkozik.
   * 
   * @param what a kiegészítendő szöveg.
   * @param extendChar a kiegészítő karakter.
   * @param length az eredményül várt szöveg hossza.
   * @return 
   */
  public static String padR(String what, String extendChar, int length) {
    String longString = repeat(extendChar.substring(0, 1), length) + what;
    return (longString).substring(longString.length() - length, longString.length());
  }

  /**
   * Adott hosszú space karakterláncot ad vissza.
   * 
   * @param length
   * @return 
   */
  public static String space(int length) {
    return repeat(" ", length);
  }

  /**
   * Megismétli az adott karakterláncot adott számszor.
   * 
   * @param what
   * @param times
   * @return 
   */
  public static String repeat(String what, int times) {
    String str = "";
    for (int c = 0; c < times; c++) {
      str += what;
    }
    return str;
  }
}
