/*
 * Szudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.Basis;

import java.awt.Color;

public interface IConst {

// -------------------------------------------------------- SWITCH or STATE

  /**
   * Konstans a bekapcsolt állapot jelöléséhez.
   */
  boolean S_ON = true;

  /**
   * Konstans a kikapcsolt állapot jelöléséhez.
   */
  boolean S_OFF = false;
    
  /**
   * Konstans az igen válaszhoz.
   */
  boolean ANS_YES = true;

  /**
   * Konstans a nem válaszhoz.
   */
  boolean ANS_NO = false;
    
  /**
   * Konstans az előre haladás jelöléséhez.
   */
  boolean W_FW = true;

  /**
   * Konstans a visszafele haladás jelöléséhez.
   */
  boolean W_BK = false;

  /**
   * Hajtsa végre a műveleteket.
   */
  boolean D_DO = true;

  /**
   * Ne hajtsa végre a műveleteket.
   */
  boolean D_DONTDO = false;

// -------------------------------------------------------- BUTTONS

  /**
   * visszalépés gomb indexe.
   */
  byte B_BACK = 11;

  /**
   * előrelépés gomb indexe.
   */
  byte B_FORWARD = 12;

  /**
   * Eredménymegjelenítő gomb indexe.
   */
  byte B_LIST = 13;
    
// -------------------------------------------------------- RESULT_TYPE
    
  /**
   * Konstans, amely jelzi, hogy az érték eredmény.
   */
  boolean T_RESULT = true;

  /**
   * Konstans, amely jelzi, hogy az érték lehetőség-érték.
   */
  boolean T_POSSIBILITY = false;

// -------------------------------------------------------- CELL_TYPE
    
  /**
   * Konstans, amely jelzi, hogy a cella jelölése
   * eredmény-értéket tároló cellát jelöl.
   */
  boolean C_RESULT = true;

  /**
   * Konstans, amely jelzi, hogy a cella jelölése
   * segédcellát jelöl.
   */
  boolean C_ASSIST = false;

// -------------------------------------------------------- AREA

  /**
   * semmivel sem kapcsolatos művelet, vagy nincs kapcsolat.
   */
  byte A_NULL = 0;

  /**
   * sorral kapcsolatos művelet, vagy azonos sor.
   */
  byte A_ROW = 1;

  /**
   * oszloppal kapcsolatos művelet, vagy azonos oszlop.
   */
  byte A_COL = 2;

  /**
   * blokkal kapcsolatos művelet, vagy azonos blokk.
   */
  byte A_BLOCK = 3;

  /**
   * cellával kapcsolatos művelet, vagy azonos cella.
   */
  byte A_CELL = 4;
    
// -------------------------------------------------------- OPERATING_MODE

  /**
   * Üzemmód, mely a képernyőn lévő cellákon
   * hajtja végre a műveleteket.
   */
  byte OP_SCREEN = 1;

  /**
   * Üzemmód, mely virtuálisan a memóriában
   * végzi a műveleteket.
   */
  byte OP_MEMORY = 2;
    
// -------------------------------------------------------- COLORS
    
  /**
   * A megoldás-érték szine.
   */
  byte COLOR_RESULT = 0;

  /**
   * A megoldást mutató segédcella szine.
   */
  byte COLOR_ASSIST_CELL = 1;
  
  /**
   * A megoldást mutató eredménycella szine.
   */
  byte COLOR_RESULT_CELL = 2;

  /**
   * A lehetőség-érték alapszine.
   */
  byte COLOR_CAND_BASE = 3;

  /**
   * A megjelölni kívánt lehetőség-érték szine.
   */
  byte COLOR_CAND_SEL = 4;

  /**
   * Rács, vagyis a cellákat tartalmazó panel szine.
   */
  byte COLOR_GRID = 5;
  
  /**
   * Azt tárolja, hány fajta olyan elem van a képernyőn,
   * aminek a színét meg lehet változtatni.
   */
  byte ELEMENTS_COUNT = 6;

// -------------------------------------------------------- COLORS AND SCHEMES

  byte SCHEME_OCEAN    = 0,
      SCHEME_NATURE   = 1,
      SCHEME_TWILIGHT = 2,
      SCHEME_VAN_GOGH = 3;

  String[] SCHEME_NAME = {"Óceán", "Természet", "Estefelé", "Van Gogh"};

  byte SCHEMES_COUNT = (byte) SCHEME_NAME.length;

  Color INDIGO = new Color(76, 122, 179);
  Color VIOLET = new Color(128, 0, 128);
  Color BROWN = new Color(128, 64, 0);

  // Result      Assist_Cell    Result_Cell   Cand_Base         Cand_Sel     Grid
  Color[][] COLOR_SCHEME = {
    {INDIGO,     Color.MAGENTA, Color.YELLOW, Color.BLUE,       Color.CYAN,  VIOLET},
    {Color.CYAN, Color.MAGENTA, Color.YELLOW, Color.RED,        Color.GREEN, Color.BLACK},
    {BROWN,      INDIGO,        Color.YELLOW, Color.LIGHT_GRAY, Color.GRAY,  Color.BLACK},
    {Color.BLUE, Color.GREEN,   Color.YELLOW, Color.ORANGE,     Color.RED,   Color.PINK}
  };

// -------------------------------------------------------- ARRAYS

  String[] OPTIONS_IGENNEM = {"Igen", "Nem"};

  String[] ROW_STRINGS = {"S Z U D O K U", "Pataki István", "06-70-298-8881", "2016", "v1.0"};

  String[] ELEMENTS = {"Eredmény-érték:", "Megoldás segéd cella:", "Megoldás eredmény cella:",
    "Lehetőség-érték alap:", "Lehetőség-érték jelölt:", "Rács:"};

  String[] OP_MODE = {"Képernyő", "Memória"};

  String[] RULESSHOW = {"", "Naked Single", "Hidden Single", "Intersection",
    "Naked Pair", "Hidden Pair", "X-Wings", "Naked Triple", "Hidden Triple",
    "Naked Quad", "Hidden Quad"};

  String[] RULESNAMES = {"", "NakedSingle", "HiddenSingle", "Intersection",
    "NakedPair", "HiddenPair", "XWings", "NakedTriple", "HiddenTriple",
    "NakedQuad", "HiddenQuad"};

  byte[] CATEGORY = {0, 1, 2, 3, 4, 2, 5, 4, 2, 4, 2};

  String[] TOOLTIPS_TEXT = {"",
    "Ez a szabály azt vizsgálja meg, hogy van e olyan cella, melyben csak egy lehetséges érték található.",
    "Megvizsgálja a cellát, hogy a munkaterületen csak ott van e az adott érték.",
    "Szegmens és sor, oszlop, vagy blokk viszonyait vizsgálja.",
    "Két olyan cellát keres, amiben csak 2 érték van, és a két érték egyenlő",
    "Két olyan cellát keres, ahol két érték csak ebben a két cellában van.",
    "X-Wings...",
    "Három olyan cellát keres, amiben csak 3 érték van, és a 3 érték egyenlő",
    "Három olyan cellát keres, ahol 3 érték csak ebben a 3 cellában van.",
    "Négy olyan cellát keres, amiben csak 4 érték van, és a 4 érték egyenlő",
    "Négy olyan cellát keres, ahol 4 érték csak ebben a 4 cellában van."};

// -------------------------------------------------------- SETTINGS DIALOG

  String[] SETTINGS_PAGES = {"Használandó szabályok", "Színek", "Egyéb"};
  byte P_UR = 0; // Használandó szabályok
  byte P_CS = 1; // Színek
  byte P_OS = 2; // Egyéb

  String[] SP_NSPOS = {"Fix első", "Változó"};

  String[] SP_WAY = {"1 => 10", "10 => 1"};

// -------------------------------------------------------- MESSAGES

  /**
   * Nincs semmi találat.
   */
  String M_EMPTY_LISTS = "A keresés egyetlen algoritmusra sem talált megoldáast.";
  
  /**
   * Teljes siker.
   */
  String M_PUZZLE_RESOLVED = "A rejtvény meg van fejtve.";

  /**
   * Teljes siker a lépések számának kiírásával.
   */
  String M_PUZZLE_RESOLVED_AND_STEPCOUNT = M_PUZZLE_RESOLVED + " Lépések száma: ";

  /**
   * A keresés elakadt.
   */
  String M_PUZZLE_NOT_RESOLVED = "Nincs további találat ...(pl. Kevés az algoritmus) !";
  
// -------------------------------------------------------- OTHERS

  /**
   * Szabályok száma.
   */
  byte RULE_COUNT = 10;
  
  /**
   * Az összes cella száma.
   */
  short CELL_COUNT = 81;
  
  /**
   * A háló mérete.
   */
  short PLAYFIELD_SIZE = 725;

  /**
   * Lépések maximális száma egy-egy rejtvényfejtés során.
   */
  short MAX_STEPS_IN_LISTS = 800;

  /**
   * A teljes jelölt értékkészlet.
   */
  String ALLCANDIDATES = "123456789";

  /**
   * Az elmentett táblák file-kiterjesztése.
   */
  String EXTENSION = "SST";  // Saved Sudoku Table 
  
  /**
   * Az ini file neve.
   */
  String INI_FILE_NAME = "SudokuPI.ini";
}
