/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.DataForms;

/**
 * Az osztály a program egy adatformáját testesíti meg.
 * A 3 területforma, a sor, az oszlop, a blokk közötti átjárhatóság
 * reprezentációja.
 */
public class DArea {

  /**
   * Screen - Memory
   */
  private byte where;

  /**
   * A művelet sorban, oszlopban, vagy blokkban történik.
   */
  private byte areaForm;

  /**
   * Hanyadik területre irányul a művelet.
   */
  private byte area;

  /**
   * Paraméterek nélküli konstruktor alapértékekkel való föltöltéshez.
   */
  public DArea() {
    this((byte) 0, (byte) 0, (byte) 0);
  }

  /**
   * Részletes paraméterezettségű konstruktor.
   * 
   * @param where
   * @param areaForm
   * @param area 
   */
  public DArea(byte where, byte areaForm, byte area) {
    setArea(where, areaForm, area);
  }

  /**
   * Alapértékek beállítása részletes paraméterezettséggel.
   * 
   * @param where
   * @param areaForm
   * @param area
   */
  public void setArea(byte where, byte areaForm, byte area) {
    setWhere(where);
    setAreaForm(areaForm);
    setArea(area);
  }

  /**
   * Alapértékek beállítása összevont paraméterrel.
   * 
   * @param area 
   */
  public void setArea(DArea area) {
    this.setArea(area.getWhere(), area.getAreaForm(), area.getArea());
  }

  /**
   * A where érték beállítása.
   * 
   * @param where 
   */
  public void setWhere(byte where) {
    this.where = where;
  }

  /**
   * Területforma beállítása.
   * 
   * @param areaForm 
   */
  public void setAreaForm(byte areaForm) {
    this.areaForm = areaForm;
  }

  /**
   * Terület sorszámának beállítása.
   * 
   * @param area 
   */
  public void setArea(byte area) {
    this.area = area;
  }

  /**
   * A where érték lekérdezése.
   * 
   * @return 
   */
  public byte getWhere() {
    return this.where;
  }

  /**
   * Területforma lekérdezése.
   * 
   * @return 
   */
  public byte getAreaForm() {
    return this.areaForm;
  }

  /**
   * Terület sorszámának lekérdezése.
   * 
   * @return 
   */
  public byte getArea() {
    return this.area;
  }

  @Override
  public String toString() {
    return "where=" + where + ", areaForm=" + areaForm + ", area=" + area;
  }

}
