package SudokuPIPkg.DataForms;

/**
 * A cella rácson belüli koordinátáját kezeli.
 * Bár akár máshova is jó lehet, semmi specialitás.
 */
public class DCoord {
  /**
   * Amióta megírtam ezt az osztályt már megtaláltam az ennek megfelelőt: Point.
   * Az is pont megfelelő lenne erre a célra. Ott x és y attribútumok vannak,
   * ami majdnem mindegy, hisz az is tükrözi a valóságot.
   */

  /**
   * sor
   */
  private byte row;

  /**
   * oszlop
   */
  private byte col;

  /**
   * Paraméterek nélküli konstruktor alapértékekkel való föltöltéshez.
   */
  public DCoord() {
    this((byte) 0, (byte) 0);
  }

  /**
   * Részletes paraméterezettségű konstruktor.
   * 
   * @param row
   * @param col 
   */
  public DCoord(byte row, byte col) {
    this.row = row;
    this.col = col;
  }

  /**
   * Összevont paraméterezettségű konstruktor.
   * 
   * @param coord 
   */
  public DCoord(DCoord coord) {
    this(coord.row, coord.col);
  }

  /**
   * Koordináta értékek beállítása részletes paraméterezettséggel.
   * 
   * @param row
   * @param col 
   */
  public void setCoord(byte row, byte col) {
    setRow(row);
    setCol(col);
  }

  /**
   * Koordináta értékek beállítása összevont paraméterezettséggel.
   * 
   * @param coord 
   */
  public void setCoord(DCoord coord) {
    this.setCoord(coord.getRow(), coord.getCol());
  }

  /**
   * Koordináta értékek lekérdezése.
   * 
   * @return 
   */
  public DCoord getCoord() {
    return new DCoord(getRow(), getCol());
  }

  /**
   * Sor érték lekérdezése.
   * 
   * @return 
   */
  public byte getRow() {
    return this.row;
  }

  /**
   * Oszlop érték lekérdezése.
   * 
   * @return 
   */
  public byte getCol() {
    return this.col;
  }

  /**
   * Sor érték beállítása.
   * 
   * @param row 
   */
  public void setRow(byte row) {
    this.row = row;
  }

  /**
   * Oszlop érték beállíítása.
   * 
   * @param col 
   */
  public void setCol(byte col) {
    this.col = col;
  }

  /**
   * Összehasonlít két koordinátát, a tároltat egy adott koordinátával.
   * 
   * @param coord
   * @return 
   */
  public boolean equals(DCoord coord) {
   /* A this.get...() visszaadja a már tárolt értékeket,
    míg a coord.get...() a beadott értéket azonosítja.*/
    return (this.getRow() == coord.getRow()) && (this.getCol() == coord.getCol());
  }

  @Override
  public String toString() {
    return "[row=" + getRow() + ", col=" + getCol() + "]";
  }

}
