/*
 * Szudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.DataForms.DNumber;
import SudokuPIPkg.DataForms.DWhere;
import SudokuPIPkg.Basis.IConst;
import SudokuPIPkg.Lists.Lists;
import SudokuPIPkg.SudokuPI;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Logger;
import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileNameExtensionFilter;
// import javax.swing.plaf.FontUIResource;
// import javax.swing.UIManager;

/**
 * A program menürendszerét kezelő osztály.
 */
public class Frame extends JFrame implements ActionListener, IConst {
  final Frame ITSME = this;
  Show myShow;
  Props storeProp;
  Lists lists;
  Rules rules;

  /**
   * Az aktuális SST tábla neve.
   */
  private String tableFileName = "";

  public JPanel pHead = new JPanel();
  public Grid pGrid;
  public JPanel pFoot = new JPanel();
  private JPanel pMenu = new JPanel(),
                 pToolBar = new JPanel(),
                 pEmptyCells = new JPanel(),
                 pStepFound = new JPanel();
  private JMenuBar mbSudoku = new JMenuBar();
  private JMenu mFile = new JMenu("Fájl"),
          mNew = new JMenu("Új");
  private JMenuItem miNewClear = new JMenuItem("Üres háló"),
                    miLoad = new JMenuItem("Betölt..."),
                    miSave = new JMenuItem("Ment..."),
                    miSettings = new JMenuItem("Beállítások..."),
                    miAboutMe = new JMenuItem("Rólam..."),
                    miClose = new JMenuItem("Bezár");
  private JToolBar tbSudoku = new JToolBar();
  private JButton[] toolBarButtons = new JButton[14];
  private JTextField tfEmptyCells = new JTextField(""),
                     tfHits = new JTextField("");
  private JLabel lEmptyCells = new JLabel("Üres cellák :"),
                 lHits = new JLabel("Talált lépés :"),
                 lFileNameLabel = new JLabel("   Fájl-név:  "),
                 lFileNameText = new JLabel("");

  /**
   * A program kezelőfelületét létrehozó és kezelő osztály konstruktora.
   */
  public Frame() {
    init();
  }

  /**
   * A főablak elemeinek elhelyezése.
   */
  private void init() {
    pHead.setLayout(new BoxLayout(pHead, BoxLayout.Y_AXIS));
    pHead.add(pMenu);
    pHead.add(pToolBar);
    myShow = new Show(ITSME);
    storeProp = new Props(ITSME, true);
    lists = new Lists();
    rules = new Rules(ITSME);
    pGrid = new Grid(ITSME);
    pGrid.setFocusable(S_ON);
    pGrid.requestFocusInWindow();
    pGrid.setEditable(S_ON);
    DCoord coord = new DCoord((byte) 4, (byte) 4);
    pGrid.changeCell(coord);
    myMenu();
    myToolBar();
    setupToolbarNumButtons(pGrid.getValidCandidates(new DWhere(OP_SCREEN, coord)));
    myFoot();
  }

  /**
   * A menüsáv kialakítását végző metódus.
   */
  private void myMenu() {
    pMenu.add(mbSudoku);
    pMenu.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    mbSudoku.add(mFile);
    mFile.add(mNew);
    mFile.add(miLoad);
    mFile.add(miSave);
    mFile.add(miSettings);
    mFile.add(new JSeparator());
    mFile.add(miAboutMe);
    mFile.add(new JSeparator());
    mFile.add(miClose);
    mNew.add(miNewClear);
    int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
    miNewClear.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, keyMask));
    miLoad.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, keyMask));
    miSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, keyMask));
    miSettings.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, keyMask));
    miAboutMe.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, keyMask));
    miClose.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, keyMask));
    storeProp.addListener(miNewClear, "File-New-Clear", this);
    storeProp.addListener(miLoad, "File-Load", this);
    storeProp.addListener(miSave, "File-Save", this);
    storeProp.addListener(miSettings, "File-Settings", this);
    storeProp.addListener(miAboutMe, "File-AboutMe", this);
    storeProp.addListener(miClose, "File-Close", this);
  }

  /**
   * Kialakítja az eszközsávot, melyen a számgombok, az előre hátra vezérlő gombok,
   * a listát megjelenítő gomb, és a táblát kiürítő gombok találhatóak.
   */
  private void myToolBar() {
    pToolBar.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
    pToolBar.add(tbSudoku);
    tbSudoku.setAlignmentX(Component.LEFT_ALIGNMENT);
    tbSudoku.setFloatable(S_OFF);
    for (int c = 0; c < 14; c++) {
      toolBarButtons[c] = new JButton();
    }
    addJButton(toolBarButtons[0], "    ", "b-0", "Értéket töröl");
    tbSudoku.addSeparator(new Dimension(5, 5));
    for (int c = 1; c <= 9; c++) {
      addJButton(toolBarButtons[c], " " + c + " ", "b-" + c, c + " érték megadása");
    }
    tbSudoku.addSeparator(new Dimension(5, 5));
    addJButton(toolBarButtons[10], "Új", "b-empty", "Új üres feladvány");
    tbSudoku.addSeparator(new Dimension(5, 5));
    addJButton(toolBarButtons[11], "<<", "b-prevstep", "Visszalép");
    addJButton(toolBarButtons[12], ">>", "b-nextstep", "Előrelép");
    tbSudoku.addSeparator(new Dimension(5, 5));
    addJButton(toolBarButtons[13], "??", "b-resultlist", "Eredménylista");
    toolBarButtons[11].setEnabled(false);
    pToolBar.add(lFileNameLabel);
    pToolBar.add(lFileNameText);
    setTableFileName("");
    showFileName();
  }

  /**
   * Elhelyez egy elemet az adott lap megfelelő pozíciójára.
   * 
   * @param comp a kihelyezendő elem
   * @param text
   * @param comm
   * @param toolTipText
   */
  private void addJButton(JButton comp, String text, String comm, String toolTipText) {
    storeProp.addListener((AbstractButton) comp, comm, this);
    comp.setToolTipText(toolTipText);
    comp.setText(text);
    comp.setFocusable(S_OFF);
    tbSudoku.add(comp);
  }

  /**
   * Kialakítja az alul lévő status-sort.
   */
  private void myFoot() {
    pFoot.setBackground(Color.LIGHT_GRAY);
    pFoot.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
    pEmptyCells.setBackground(Color.LIGHT_GRAY);
    pEmptyCells.setForeground(Color.CYAN);
    pFoot.add(pEmptyCells);
    lEmptyCells.setBackground(Color.LIGHT_GRAY);
    lEmptyCells.setHorizontalAlignment(SwingConstants.LEFT);
    lEmptyCells.setFont(new Font("Tahoma", Font.BOLD, 9));
    pEmptyCells.add(lEmptyCells);
    tfEmptyCells.setHorizontalAlignment(SwingConstants.RIGHT);
    tfEmptyCells.setFont(new Font("Tahoma", Font.BOLD, 9));
    tfEmptyCells.setEditable(false);
    tfEmptyCells.setColumns(3);
    pEmptyCells.add(tfEmptyCells);
    pStepFound.setBackground(Color.LIGHT_GRAY);
    pFoot.add(pStepFound);
    lHits.setHorizontalAlignment(SwingConstants.LEFT);
    lHits.setFont(new Font("Tahoma", Font.BOLD, 9));
    pStepFound.add(lHits);
    tfHits.setToolTipText("");
    tfHits.setHorizontalAlignment(SwingConstants.LEFT);
    tfHits.setFont(new Font("Tahoma", Font.BOLD, 9));
    tfHits.setEditable(false);
    tfHits.setColumns(45);
    pStepFound.add(tfHits);
    showEmptyCells(OP_SCREEN, pGrid.getEmptyCellsCounterString(OP_SCREEN));
  }

  // --------------------------------------------------------------------------- Eseménykezelők
  
  /**
   * A menüpontok és toolbargombok eseménykezelöje.
   *
   * @param ae ActionEvent
   */
  @Override
  public void actionPerformed(ActionEvent ae) {
    String sender = ae.getActionCommand();
    switch (sender) {
      case "File-New-Clear":
      case "b-empty":
        clearTable();
        break;
      case "File-Load":
        myShow.setShowStarted(ANS_NO);
        storeProp.setupPages();
        try {
          loadTable();
          pGrid.setEditable(S_ON);
        } catch (IOException ioEx) {
          Logger.getLogger(sender);
        }
        lists.clearAssistAndResultLists();
        pGrid.setActCell(new DCoord((byte) 0, (byte) 0));
        DCoord coord2 = new DCoord((byte) 4, (byte) 4);
        pGrid.changeCell(coord2);
        setupToolbarNumButtons(pGrid.getValidCandidates(new DWhere(OP_SCREEN, coord2)));
        setToolbarButtonEnabled(B_BACK, S_OFF);
        setToolbarButtonEnabled(B_FORWARD, S_ON);
        break;
      case "File-Save":
        try {
          saveTable();
        } catch (IOException ioEx) {
          Logger.getLogger(sender);
        }
        break;
      case "File-Settings":
        myShow.setShowStarted((lists.getStepCount() > 0 ? ANS_YES : ANS_NO));
        storeProp.setupPages();
        storeProp.showPropsFrame();
        if (storeProp.isColorSchemeChanged()) {
          pGrid.repaintAllColors();
        }
        break;
      case "File-AboutMe":
        JOptionPane.showMessageDialog(SudokuPI.getpSudoku(), new AboutMe(),
                "Rólam ...", JOptionPane.PLAIN_MESSAGE);
        break;
      case "File-Close":
        System.exit(0);
        break;
      case "b-0":
      case "b-1":
      case "b-2":
      case "b-3":
      case "b-4":
      case "b-5":
      case "b-6":
      case "b-7":
      case "b-8":
      case "b-9":
        pGrid.numPressed(sender.substring(2));
        break;
      case "b-nextstep":
        myShow.setShowStarted(ANS_YES);
        storeProp.setupPages();
        if (storeProp.getOperatingModeFromProp() == OP_MEMORY) {
          pGrid.setEditable(S_OFF);
          pGrid.setCellMark(pGrid.getActCell(), T_RESULT, S_OFF);
        }
        myShow.stepForward(storeProp.getOperatingModeFromProp());
        break;
      case "b-prevstep":
        myShow.stepBack();
        break;
      case "b-resultlist":
        showResults();
        break;
      default: throw new IllegalArgumentException("Hiba: 1234817236418273");
    }
  }

  // --------------------------------------------------------------------------- Menüpontokhoz tartozó metódusok

  /**
   * Képernyő-tábla kitakarítása.
   */
  private void clearTable() {
    myShow.setShowStarted(ANS_NO);
    storeProp.setupPages();
    setTableFileName("");
    showFileName();
    pGrid.resetTable(OP_SCREEN, S_ON);
    setupFrame();
    pGrid.setEditable(S_ON);
    lists.clearAssistAndResultLists();
    pGrid.setPrevCell(new DCoord((byte) 0, (byte) 0));
    DCoord coord = new DCoord((byte) 4, (byte) 4);
    pGrid.changeCell(coord);
    setupToolbarNumButtons(pGrid.getValidCandidates(new DWhere(OP_SCREEN, coord)));
    showResultText(OP_SCREEN, "Tábla üres");   
  }

  /**
   * Megjeleníti a bemutatóhoz tartozó eredménylistát.
   */
  private void showResults() {
    JPanel pList = new JPanel();
    BorderLayout loListPanel = new BorderLayout();
    pList.setLayout(loListPanel);
    JScrollPane spList = new JScrollPane();
    JTextArea taLines = new JTextArea();
    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    taLines.setFont(new Font("Tahoma", Font.PLAIN, 16));
    spList.setViewportView(taLines);
    byte vBar = JScrollPane.VERTICAL_SCROLLBAR_ALWAYS;
    byte hBar = JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED;
    JScrollPane spScroll = new JScrollPane(taLines, vBar, hBar);
    pList.add(BorderLayout.CENTER, spScroll);
    Dimension sizeListPanel = new Dimension(500, 500);
    pList.setPreferredSize(sizeListPanel);
    pList.setMaximumSize(sizeListPanel);
    pList.setSize(sizeListPanel);
    taLines.setEditable(S_OFF);
    taLines.setText(lists.toString());
    taLines.selectAll();
    clipboard.setContents(new StringSelection(taLines.getSelectedText()), null);
    taLines.select(0, 0);
    JPanel pane = SudokuPI.getpSudoku();
    String text = "Az eredménylista tartalma:";
    JOptionPane.showMessageDialog(pane, pList, text, JOptionPane.PLAIN_MESSAGE);
  }

  /**
   * Tábla betöltése.
   *
   * @throws java.io.IOException
   */
  private void loadTable() throws java.io.IOException {
    String fRow;
    boolean correct = true;
    byte row;
    JFileChooser fc = new JFileChooser();
    fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    String ext = storeProp.getExtention(false);
    FileNameExtensionFilter filesFilter = 
            new FileNameExtensionFilter("Saved Sudoku Table", ext);
    fc.setFileFilter(filesFilter);
    File dir = new File("Saves");
    if (!dir.exists()) {
      dir.mkdir();
    }
    fc.setCurrentDirectory(dir);
    int rv = fc.showOpenDialog(SudokuPI.getpSudoku());
    if (rv == JFileChooser.APPROVE_OPTION) {
      File selectedFile = fc.getSelectedFile();
      setTableFileName(selectedFile.getName());
      showFileName();
      try
        (BufferedReader br = new BufferedReader(new FileReader(selectedFile))) {
        row = 0;
        fRow = br.readLine();
        pGrid.resetTable(OP_MEMORY, S_OFF);
        setupFrame();
        while ((fRow != null) && (correct)) {
          // egy sor feldolgozása
          String[] cols = convertRowToCells(fRow);
          correct = (cols != null);
          if (correct) {
            for (byte col = 0; col < 9; col++) {
              DWhere where = new DWhere(OP_MEMORY, new DCoord(row, col));
              pGrid.setCell(new DNumber(where, (byte) 0), cols[col]);
            }
            row++;
            fRow = br.readLine();
          }
        }
        br.close();
        if (correct) {
          pGrid.showTable(OP_SCREEN);
          setupFrame();
          showResultText(OP_SCREEN, "Tábla betöltve.");
        } else {
          showResultText(OP_SCREEN, "Hibás a mentett tábla!");
        }
      }
    }
    DCoord coord = new DCoord((byte) 4, (byte) 4);
    pGrid.changeCell(coord);
    setupToolbarNumButtons(pGrid.getValidCandidates(new DWhere(OP_SCREEN, coord)));
    pGrid.setEditable(S_ON);
  }

  /**
   * Egy sort tartalmazó stringet cellákra bontja.
   * 
   * @param fRow a cellákat tartalmazó teljes sor
   * @return 
   */
  private String[] convertRowToCells(String fRow) {
    String[] cells = new String[9];
    int c = 0;
    byte col = 0;
    String actChar;
    while ((c < fRow.length()) && (col < 9)) {
      actChar = fRow.substring(c, c + 1);
      if (actChar.equals("{")) {
        String cell = "{";
        c++;
        actChar = fRow.substring(c, c + 1);
        while (!actChar.equals("}")) {
          cell += actChar;
          c++;
          actChar = fRow.substring(c, c + 1);
        }
        cells[col] = cell + "}";
      } else {
        cells[col]= actChar;
      }
      col++;
      c++;
    }
    return (col < 9 ? null : cells);
  }

  /**
   * Tábla mentése.
   *
   * @throws java.io.IOException
   */
  private void saveTable() throws java.io.IOException {
    String sff = "";
    String fRow, cell;
    Date now = new Date();
    DateFormat df = new SimpleDateFormat("yyyyMMdd-HHmmss");
    JFileChooser fc = new JFileChooser();
    File dir = new File("Saves");
    String ext = storeProp.getExtention(false);
    FileNameExtensionFilter filesFilter =
            new FileNameExtensionFilter("Saved Sudoku Table", ext);
    fc.setFileFilter(filesFilter);
    if (!dir.exists()) {
      dir.mkdir();
    }
    fc.setCurrentDirectory(dir);
    fc.setSelectedFile(new File(df.format(now) + storeProp.getExtention(true)));
    int rv = fc.showSaveDialog(SudokuPI.getpSudoku());
    if (rv == JFileChooser.APPROVE_OPTION) {
      String sf = fc.getSelectedFile().getName();
      if (sf.length() < 4) {
        sf += storeProp.getExtention(true);
      }
      if (!sf.substring(sf.length() - 4).equals(storeProp.getExtention(true))) {
        sf += storeProp.getExtention(true);
      }
      setTableFileName(sf);
      File f = new File(dir, sf);
      if (f.exists()) {
        f.delete();
      }
      try (BufferedWriter bw = new BufferedWriter(new FileWriter(f))) {
        for (byte row = 0; row < 9; row++) {
          fRow = "";
          for (byte col = 0; col < 9; col++) {
            cell = pGrid.getCell(new DWhere(OP_SCREEN, new DCoord(row, col)));
            fRow += cell;
          }
          bw.write(fRow);
          bw.newLine();
          bw.flush();
        }
        bw.close();
      }
      showFileName();
      showResultText(OP_SCREEN, "Tábla elmentve.");
    }
  }

  // --------------------------------------------------------------------------- Képernyő elemek aktualizátorai

  /**
   * Engedélyezi / letiltja az adott sorszámú eszköz-gombot.
   *
   * @param ord hányas sorszámú gombot állítsa be.
   * @param state milyen állapotra.
   */
  public void setToolbarButtonEnabled(byte ord, boolean state) {
    toolBarButtons[ord].setEnabled(state);
    toolBarButtons[ord].repaint();
  }

  /**
   * Beállítja a számgombokat az aktuális cella alapján.
   * 
   * @param candidates
   */
  public void setupToolbarNumButtons(String candidates) {
    setToolbarButtonEnabled((byte) 0, candidates.isEmpty());
    for (byte candidate = 1; candidate < 10; candidate++) {
      setToolbarButtonEnabled(candidate, candidates.contains("" + candidate));
    }
  }

// ----------------------------------------------------------------------------- Tábla-név kezelése

  /**
   * Beállítja a megjelenő file nevét a kezelőfelületen.
   */
  private void showFileName() {
    lFileNameLabel.setVisible(!getTableFileName().isEmpty());
    lFileNameText.setVisible(!getTableFileName().isEmpty());
    lFileNameText.setText((getTableFileName().length() < 32)
            ? getTableFileName() : getTableFileName().substring(0, 30));
  }

  /**
   * Visszaadja a mentett tábla file nevét.
   * 
   * @return
   */
  public String getTableFileName() {
    return tableFileName;
  }

  /**
   * Beállítja a mentett tábla file nevét.
   * 
   * @param fileName 
   */
  private void setTableFileName(String fileName) {
    this.tableFileName = fileName;
  }

  /**
   * A talált lépés mezőjének tooltip szövegét állítja be.
   *
   * @param rule a szabály sorszáma
   */
  public void showHitsToolTipText(byte rule) {
    tfHits.setToolTipText(TOOLTIPS_TEXT[rule]);
  }

  /**
   * Megjeleníti az üres cellák számát.
   *
   * @param where hova<br> &nbsp;&nbsp;&nbsp - OP_SCREEN : Akkor a szövegmezöt
   * tölti fel az értékkel.<br> &nbsp;&nbsp;&nbsp - OP_MEMORY : A Console
   * képernyöre írja ki az értéket.
   * @param number Ures cellák száma
   */
  public void showEmptyCells(byte where, String number) {
    if (where == OP_SCREEN) {
      tfEmptyCells.setText(number);
      tfEmptyCells.repaint();
    } else {
      System.out.println("Üres cellák száma a tömbben : " + number);
    }
  }

  /**
   * Megjeleníti az aktuális megoldás szövegét
   *
   * @param where hova<br> &nbsp;&nbsp;&nbsp - OP_SCREEN : Akkor a szövegmezöt
   * tölti fel az értékkel.<br> &nbsp;&nbsp;&nbsp - OP_MEMORY : A Console
   * képernyöre írja ki az értéket.
   * @param text A megoldó függvény neve és további információk
   */
  public void showResultText(byte where, String text) {
    if (where == OP_SCREEN) {
      tfHits.setText(text);
      tfHits.repaint();
    } else {
      System.out.println("A megoldó-függvény : " + text);
    }
  }

  /**
   * Frissíti a kezelőfelületet.
   */
  public void setupFrame() {
    DWhere where = new DWhere(OP_SCREEN, pGrid.getActCell());
    setupToolbarNumButtons(pGrid.getValidCandidates(where));
    showEmptyCells(OP_SCREEN, pGrid.getEmptyCellsCounterString(OP_SCREEN));
  }

}
