/*
 * Szudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.Basis.Basics;
import SudokuPIPkg.DataForms.*;
import SudokuPIPkg.Basis.IConst;
import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.border.TitledBorder;

/**
 * A program kezelőfelületét létrehozó és kezelő osztály.
 */
public class Grid extends JPanel implements KeyListener, IConst {
  
  private Frame parent = null;
  private JPopupMenu popup = new JPopupMenu();
  private JMenuItem[] popupItem = new JMenuItem[10];
  private JList itemList = new JList();
  private DCoord[][][] workAreas = new DCoord[3][9][9];
  private DColors actColorSet = new DColors();

  private Cell[][] screenCells = new Cell[9][9];
  private String[][] virtualCells = new String[9][9];
  private short emptyVirtualCells = CELL_COUNT,
              emptyScreenCells = CELL_COUNT;
  private boolean editable = ANS_YES;
  private DCoord actCell, prevCell;

  /**
   * A program kezelőfelületét létrehozó és kezelő osztály konstruktora.
   * @param owner
   */
  public Grid(Frame owner) {
    super(null);
    if (owner != null) {
      this.parent = owner;
      init();
    }
  }

  /**
   * Kialakítja a játékteret.
   */
  private void init() {
    actCell = new DCoord((byte) 0, (byte) 0);
    prevCell = new DCoord((byte) 0, (byte) 0);
    workAreas = new DCoord[3][9][9];
    fillIndexTable();
    int cell_row, cell_col;
    cell_row = 4;
    for (byte row = 0; row < 9; row++) {
      cell_col = 3;
      for (byte col = 0; col < 9; col++) {
        screenCells[row][col] = new Cell(new DCoord(row, col), actColorSet);
        screenCells[row][col].setBackground(Color.WHITE);
        screenCells[row][col].setBounds(cell_col, cell_row, 65, 65);
        screenCells[row][col].addMouseListener(new CellListener());
        screenCells[row][col].setFocusable(S_ON);
        add(screenCells[row][col]);
        cell_col = cell_col + 66;
        if (col == 2 || col == 5) {
          cell_col = cell_col + 3;
        }
      }
      cell_row = cell_row + 66;
      if (row == 2 || row == 5) {
        cell_row = cell_row + 3;
      }
    }
    setSize(PLAYFIELD_SIZE, PLAYFIELD_SIZE);
    setLayout(null);
    addKeyListener(this);
    setFocusable(S_ON);
    actColorSet = parent.storeProp.getActualColors();
    repaintAllColors();
    requestFocusInWindow();
    setEditable(S_ON);
    changeCell(new DCoord((byte) 4, (byte) 4));
    myPopup();
  }

  /**
   * Itt a cellákhoz tartozó popup menüt hozom létre.
   * Az összes értéket felveszem a listába, majd megjelenítés előtt
   * beállítom, hogy melyik legyen látható.
   */
  private void myPopup() {
    popup = new JPopupMenu();
    popupItem = new JMenuItem[10];
    for (byte c = 0; c < 10; c++) {
      popup.add(popupItem[c] = new JMenuItem(c==0 ? "töröl" : "  " + c));
      parent.storeProp.addListener(popupItem[c], "b-" + c, parent);
    }
    popup.setBorder(new TitledBorder(""));
    itemList.add(popup);
  }

  /**
   * Átszinezi a játékteret.
   */
  public void repaintAllColors() {
    setBackground(parent.storeProp.getActualColor(COLOR_GRID));
    for (byte row = 0; row < 9; row++) {
      for (byte col = 0; col < 9; col++) {
        screenCells[row][col].repaintColors(parent.storeProp.getActualColors());
      }
    }
    if (parent.myShow.isShowStarted() == ANS_YES) {
      parent.myShow.assistsSetShow(S_ON);
    }
  }

  // --------------------------------------------------------------------------- Eseménykezelők
  
  /**
   * A popup-menü menüpontjait állítja be a Toolbar gombjai alapján,
   * és megjeleníti azt.
   * 
   * @param me 
   */
  public void setupAndShowPopup(MouseEvent me) {
    DWhere where = new DWhere(OP_SCREEN, getActCell());
    popupItem[0].setVisible(isCellResolved(where));
    for (byte candidate = 1; candidate < 10; candidate++) {
      DNumber value = new DNumber(where, candidate);
      popupItem[candidate].setVisible(isCandidateStateOn(value));
    }
    if (me == null) {
      popup.show((JPanel) screenCells[where.getRow()][where.getCol()], 33, 33);
    } else {
      popup.show((JPanel) me.getSource(), me.getX(), me.getY());
    }
  }

  /**
   * Billentyűzet eseménykezelője Sztringes paraméterrel
   * 
   * @param actValue 
   */
  public void numPressed(String actValue) {
    numPressed(Byte.parseByte(actValue));
  }
  
  /**
   * Billentyűzet eseménykezelője Egész-szám paraméterrel
   * 
   * @param actValue 
   */
  public void numPressed(byte actValue) {
    DCoord coord = getActCell();
    String pos1, pos2;
    byte blockNumber = Basics.getBlocknumberFromPosition(coord);
    if (actValue > 0) {
      screenCells[coord.getRow()][coord.getCol()].setResult(actValue);
      for (byte c = 0; c < 9; c++) {
        screenCells[coord.getRow()][c].setCandidateState(actValue, S_OFF);
        screenCells[c][coord.getCol()].setCandidateState(actValue, S_OFF);
        byte row = Basics.getCoordFromBlockPosition(blockNumber, c).getRow();
        byte col = Basics.getCoordFromBlockPosition(blockNumber, c).getCol();
        screenCells[row][col].setCandidateState(actValue, S_OFF);
      }
      setEmptyCellsCounterByAdding(OP_SCREEN, (short) -1);
      parent.showResultText(OP_SCREEN, actValue + " érték megadva");
    } else {
      DArea area;
      byte row = coord.getRow();
      byte col = coord.getCol();
      String result = getResult(new DWhere(OP_SCREEN, coord));
      byte iResult = Byte.parseByte(result);
      clearResult(new DWhere(OP_SCREEN, coord));
      String sRow, sCol, sBlock;
      for (byte c = 1; c < 10; c++) {
        area = new DArea(OP_SCREEN, A_ROW, row);
        sRow = getResultPositionsInArea(area, "" + c);
        area = new DArea(OP_SCREEN, A_COL, col);
        sCol = getResultPositionsInArea(area, "" + c);
        area = new DArea(OP_SCREEN, A_BLOCK, blockNumber);
        sBlock = getResultPositionsInArea(area, "" + c);
        if ((sRow + sCol + sBlock).length() > 0) {
          DNumber number = new DNumber(new DWhere(OP_SCREEN, coord), c);
          setCandidateState(new DState(number, S_OFF));
        }
      }
      for (byte col2 = 0; col2 < 9; col2++) {
        byte blockNumber2 = Basics.getBlocknumberFromPosition(new DCoord(row, col2));
        pos1 = getResultPositionsInArea(new DArea(OP_SCREEN, A_COL, col2), result);
        area = new DArea(OP_SCREEN, A_BLOCK, blockNumber2);
        pos2 = getResultPositionsInArea(area, result);
        if ((pos1 + pos2).isEmpty()) {
          unclearCandidate(new DCoord(row, col2), iResult);
        }
      }
      for (byte row2 = 0; row2 < 9; row2++) {
        byte blockNumber2 = Basics.getBlocknumberFromPosition(new DCoord(row2, col));
        pos1 = getResultPositionsInArea(new DArea(OP_SCREEN, A_ROW, row2), result);
        area = new DArea(OP_SCREEN, A_BLOCK, blockNumber2);
        pos2 = getResultPositionsInArea(area, result);
        if ((pos1 + pos2).isEmpty()) {
          unclearCandidate(new DCoord(row2, col), iResult);
        }
      }
      byte boY = (byte) ((blockNumber / 3) * 3);
      byte boX = (byte) ((blockNumber % 3) * 3);
      for (byte offsetY = 0; offsetY < 3; offsetY++) {
        for (byte offsetX = 0; offsetX < 3; offsetX++) {
          area = new DArea(OP_SCREEN, A_ROW, (byte) (boY + offsetY));
          pos1 = getResultPositionsInArea(area, result);
          area = new DArea(OP_SCREEN, A_COL, (byte) (boX + offsetX));
          pos2 = getResultPositionsInArea(area, result);
          if ((pos1 + pos2).isEmpty()) {
            byte row2 = (byte) (boY + offsetY);
            byte col2 = (byte) (boX + offsetX);
            unclearCandidate(new DCoord(row2, col2), iResult);
          }
        }
      }
      setEmptyCellsCounterByAdding(OP_SCREEN, (short) 1);
      parent.showResultText(OP_SCREEN, result + " érték törölve.");
    }
    if (isEditable()) {
      parent.showEmptyCells(OP_SCREEN, "" + getEmptyCellsCounterString(OP_SCREEN));
      changeCell(coord);
    }
    parent.setupToolbarNumButtons(getValidCandidates(new DWhere(OP_SCREEN, coord)));
  }

  /**
   * Jelöltértéket állítja vissza
   * 
   * @param coord
   * @param iResult 
   */
  private void unclearCandidate(DCoord coord, byte iResult) {
    if (!isCellResolved(new DWhere(OP_SCREEN, coord))) {
      DNumber number = new DNumber(new DWhere(OP_SCREEN, coord), iResult);
      setCandidateState(new DState(number, S_ON));
    }
  }
  
  @Override
  public void keyTyped(KeyEvent ke) {
    char sChar = ke.getKeyChar();
    if((sChar >= '0') && (sChar <= '9')) {
      numPressed((byte) (sChar - '0'));
    }
  }

  @Override
  public void keyPressed(KeyEvent ke) {
    byte posY = getActCell().getRow();
    byte posX = getActCell().getCol();
    int sender = ke.getKeyCode();
    boolean changedPosition = ANS_NO;
    switch (sender) {
      case KeyEvent.VK_LEFT :
        posX = (posX > 0 ? --posX : 8);
        changedPosition = ANS_YES;
        break;
      case KeyEvent.VK_RIGHT :
        posX = (posX < 8 ? ++posX : 0);
        changedPosition = ANS_YES;
        break;
      case KeyEvent.VK_UP :
        posY = (posY > 0 ? --posY : 8);
        changedPosition = ANS_YES;
        break;
      case KeyEvent.VK_DOWN :
        posY = (posY < 8 ? ++posY : 0);
        changedPosition = ANS_YES;
        break;
      case KeyEvent.VK_ENTER :
        setupAndShowPopup(null);
      default:
    }
    if (changedPosition == ANS_YES) {
      DCoord coord = new DCoord(posY, posX);
      changeCell(coord);
      parent.setupToolbarNumButtons(getValidCandidates(new DWhere(OP_SCREEN, coord)));
    }
  }

  @Override
  public void keyReleased(KeyEvent ke) {
  }

  /**
   * A képernyörács eseménykezelöje, ennek segítségével határozom meg, hogy
   * melyik cella lett kiválasztva.
   */
  private class CellListener extends MouseAdapter {
    @Override
    public void mouseClicked(MouseEvent me) {
      boolean cond1 = me.getButton() == MouseEvent.BUTTON1;
      if (cond1 || (me.getButton() == MouseEvent.BUTTON3)) {
        Cell clickedCell = (Cell) me.getSource();
        if (isEditable()) {
          DCoord coord = new DCoord(clickedCell.getCoord());
          changeCell(coord);
          parent.setupToolbarNumButtons(getValidCandidates(new DWhere(OP_SCREEN, coord)));
        }
        if (me.getButton() == MouseEvent.BUTTON3) {
          setupAndShowPopup(me);
        }
      }
    }
  }

  // --------------------------------------------------------------------------- Cellák tartalmának kezelői

  /**
   * Az adott cellában törli az eredményértéket.
   *
   * @param where
   */
  public void clearResult(DWhere where) {
    setCell(where, "{" + ALLCANDIDATES + "}");
  }

  /**
   * A lehetöségérték állapotát adja vissza.
   *
   * @param number
   * @return
   */
  public boolean isCandidateStateOn(DNumber number) {
    DWhere where = new DWhere(number.getWhere(), new DCoord(number.getCoord()));
    boolean cond1 = getCell(where).contains("" + number.getNumber());
    return cond1 && !isCellResolved(where);
  }

  /**
   * A lehetöségértékek számát adja vissza. tuti jól működik
   *
   * @param where
   * @return
   */
  public byte getCandidatesCounterInCell(DWhere where) {
    return (byte) getValidCandidates(where).length();
  }

  /**
   * Visszaadja, hogy az adott területen hol található az adott jelöltérték.
   *
   * @param area hanyadik ~
   * @param candidate lehetĹ‘ségérték
   * @return
   */
  public String getCandidatePositionsInArea(DArea area, byte candidate) {
    byte aWhere = area.getWhere();
    byte boY, boX;
    DWhere where;
    String positions = "";
    if (area.getAreaForm() == A_ROW) {
      for (byte c = 0; c < 9; c++) {
        where = new DWhere(aWhere, new DCoord(area.getArea(), c));
        if (isCandidateStateOn(new DNumber(where, candidate))) {
          positions += c;
        }
      }
    } else if (area.getAreaForm() == A_COL) {
      for (byte c = 0; c < 9; c++) {
        where = new DWhere(aWhere, new DCoord(c, area.getArea()));
        if (isCandidateStateOn(new DNumber(where, candidate))) {
          positions += c;
        }
      }
    } else {
      boY = (byte) ((area.getArea() / 3) * 3);
      boX = (byte) ((area.getArea() % 3) * 3);
      for (byte offsetY = 0; offsetY < 3; offsetY++) {
        for (byte offsetX = 0; offsetX < 3; offsetX++) {
          byte row = (byte) (boY + offsetY);
          byte col = (byte) (boX + offsetX);
          where = new DWhere(aWhere, new DCoord(row, col));
          if (isCandidateStateOn(new DNumber(where, candidate))) {
            positions += (offsetY * 3) + offsetX;
          }
        }
      }
    }
    return (positions);
  }

  /**
   * Visszaadja egy adott cella tartalmát
   *
   * @param where
   * @return
   */
  public String getCell(DWhere where) {
    byte row = where.getCoord().getRow();
    byte col = where.getCoord().getCol();
    if (where.getWhere() == OP_SCREEN) {
      return screenCells[row][col].toString();
    } else {
      return virtualCells[row][col];
    }
  }

  /**
   * A területen belüli pozícióból valódi koordinátát állít elő.
   * 
   * @param aPos 
   * @return 
   */
  public DCoord getCoordFromPosition(DAPos aPos) {
    return (workAreas[aPos.getAreaForm()][aPos.getArea()][aPos.getOffset()]);
  }

  /**
   * Megszámolja, hogy az adott területen a jelölt hányszor szerepel.
   *
   * @param area 
   * @param candidate jelöltérték
   * @return
   */
  public byte getCountCandidateInArea(DArea area, byte candidate) {
    byte aWhere = area.getWhere();
    DWhere where;
    byte boY, boX, count = 0;
    switch (area.getAreaForm()) {
      case A_ROW :
        for (byte c = 0; c < 9; c++) {
          where = new DWhere(aWhere, new DCoord(area.getArea(), c));
          if (isCandidateStateOn(new DNumber(where, candidate))) {
            count++;
          }
        }
      case A_COL :
        for (byte c = 0; c < 9; c++) {
          where = new DWhere(aWhere, new DCoord(c, area.getArea()));
          if (isCandidateStateOn(new DNumber(where, candidate))) {
            count++;
          }
        }
      default :
        boY = (byte) ((area.getArea() / 3) * 3);
        boX = (byte) ((area.getArea() % 3) * 3);
        for (byte offsetY = 0; offsetY < 3; offsetY++) {
          for (byte offsetX = 0; offsetX < 3; offsetX++) {
            byte row = (byte) (boY + offsetY);
            byte col = (byte) (boX + offsetX);
            where = new DWhere(aWhere, new DCoord(row, col));
            if (isCandidateStateOn(new DNumber(where, candidate))) {
              count++;
            }
          }
        }
    }
    return (count);
  }

  /**
   * Összeszámolja az üres cellákat.
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @return
   */
  public short recountEmptyCells(byte where) {
    short counter = 0;
    for (byte row = 0; row < 9; row++) {
      for (byte col = 0; col < 9; col++) {
        if (!isCellResolved(new DWhere(where, new DCoord(row, col)))) {
          counter++;
        }
      }
    }
    return counter;
  }

  /**
   * Visszaadja, hogy az adott cella meg van-e fejtve.
   *
   * @param where
   * @return
   */
  public boolean isCellResolved(DWhere where) {
    return !getCell(where).substring(0, 1).equals("{");
  }

  /**
   * Az adott cella eredményértékét adja vissza.
   * Ha a visszaadott érték üres sztring, a cella nincs megfejtve.
   *
   * @param where
   * @return
   */
  public String getResult(DWhere where) {
    return (isCellResolved(where) ? getCell(where) : "");
  }

  /**
   * Visszaadja, hogy az adott eredményérték mely pozíciókon szerepel az adott
   * területen.
   *
   * @param area 
   * @param result a keresendö érték
   * @return
   */
  public String getResultPositionsInArea(DArea area, String result) {
    int sm = area.getWhere();
    String positions = "";
    if (area.getAreaForm() == A_ROW) {
      for (byte col = 0; col < 9; col++) {
        if (getResult(new DWhere((byte) sm, new DCoord(area.getArea(), col))).equals(result)) {
          positions += "" + col;
        }
      }
    } else if (area.getAreaForm() == A_COL) {
      for (byte row = 0; row < 9; row++) {
        if (getResult(new DWhere((byte) sm, new DCoord(row, area.getArea()))).equals(result)) {
          positions += "" + row;
        }
      }
    } else {
      byte boY = (byte) ((area.getArea() / 3) * 3);
      byte boX = (byte) ((area.getArea() % 3) * 3);
      for (int offsetY = 0; offsetY < 3; offsetY++) {
        for (int offsetX = 0; offsetX < 3; offsetX++) {
          byte row = (byte) (boY + offsetY);
          byte col = (byte) (boX + offsetX);
          DCoord coord = new DCoord(row, col);
          if (getResult(new DWhere((byte) sm, coord)).equals(result)) {
            positions += "" + ((offsetY * 3) + offsetX);
          }
        }
      }
    }
    return positions;
  }

  /**
   * Visszaadja egy sztringben a cella lehetöségértékeit.
   *
   * @param where
   * @return
   */
  public String getValidCandidates(DWhere where) {
    if (isCellResolved(where)) {
      return "";
    } else {
      String cell = getCell(where);
      return cell.substring(1, cell.length() - 1);
    }
  }

  /**
   * Egy jelölt 'area'-edik elöfordulásának pozícióját adja vissza az adott
   * területen.
   *
   * @param area hanyadik terült
   * @param offsetOrd hanyadik pozícióra van szükségem
   * @param candidate lehetöségérték
   * @return
   */
  public byte getPositionInArea(DArea area, byte offsetOrd, byte candidate) {
    byte aWhere = area.getWhere();
    DWhere where;
    byte boY, boX, offsetY, offsetX;
    byte position = 0, ordinal = 0;
    if (area.getAreaForm() == A_ROW) {
      while ((position < 9) && (ordinal < offsetOrd)) {
        where = new DWhere(aWhere, new DCoord(area.getArea(), position));
        if (isCandidateStateOn(new DNumber(where, candidate))) {
          ordinal++;
        }
        position++;
      }
    } else if (area.getAreaForm() == A_COL) {
      while ((position < 9) && (ordinal < offsetOrd)) {
        where = new DWhere(aWhere, new DCoord(position, area.getArea()));
        if (isCandidateStateOn(new DNumber(where, candidate))) {
          ordinal++;
        }
        position++;
      }
    } else {
      boY = (byte) ((area.getArea() / 3) * 3);
      boX = (byte) ((area.getArea() % 3) * 3);
      while ((position < 9) && (ordinal < offsetOrd)) {
        offsetY = (byte) (position / 3);
        offsetX = (byte) (position % 3);
        byte row = (byte) (boY + offsetY);
        byte col = (byte) (boX + offsetX);
        where = new DWhere(aWhere, new DCoord(row, col));
        if (isCandidateStateOn(new DNumber(where, candidate))) {
          ordinal++;
        }
        position++;
      }
    }
    return (byte) (position - 1);
  }

  /**
   * Visszaadja, hogy a tábla megvan-e fejtve.
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @return
   */
  public boolean isTableResolved(byte where) {
    return recountEmptyCells(where) == 0;
  }

  /**
   * Visszaadja, hogy a tábla láthatólag helyes-e.
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @return
   */
  public boolean isTableCorrect(byte where) {
    boolean correct = true;
    boolean cond1, cond2;
    DWhere dWhere;
    for (byte row = 0; row < 9; row++) {
      for (byte col = 0; col < 9; col++) {
        dWhere = new DWhere(where, new DCoord(row, col));
        cond1 = (!isCellResolved(dWhere));
        cond2 = (getCandidatesCounterInCell(dWhere) == 0);
        if (cond1 && cond2) {
          correct = false;
        }
      }
    }
    return correct;
  }

  /**
   * Alaphelyzetbe állítja a táblát.
   *
   * @param where hol töröljön (OP_SCREEN - OP_MEMORY)
   * @param stateCandidates erre az állapotra állítja be a jelölteket.
   */
  public void resetTable(byte where, boolean stateCandidates) {
    DCoord coord;
    DNumber number;
    for (byte row = 0; row < 9; row++) {
      for (byte col = 0; col < 9; col++) {
        coord = new DCoord(row, col);
        DWhere dWhere = new DWhere(where, new DCoord(row, col));
        setCell(dWhere, (stateCandidates ? ("{" + ALLCANDIDATES + "}") : ""));
        if (where == OP_SCREEN) { // mindenféle jelölést is töröljön.
          setCellMark(coord, T_RESULT, S_OFF);
          for (byte cand = 1; cand <= 9; cand++){
            number = new DNumber(new DWhere(where, coord), cand);
            setCandidateMark(new DState(number, S_OFF));
          }
        }
      }
    }
    setEmptyCellsCounter(where, (stateCandidates ? CELL_COUNT : 0));
    if (where == OP_SCREEN) {
      setEditable(S_ON);
      coord = new DCoord((byte) 4, (byte) 4);
      changeCell(coord);
      parent.setupFrame();
    }
  }

  /**
   * Cellaváltásnál beállítja az elözö lépés cellapozícióját, majd vált az újra.
   *
   * @param coord
   */
  public void changeCell(DCoord coord) {
    setPrevCell(getActCell());
    setActCell(coord);
    setCellMark(getPrevCell(), C_RESULT, S_OFF);
    setCellMark(getActCell(), C_RESULT, isEditable());
  }
  
  /**
   * Az adott cella háttérszínét állítja be az adott színre.
   *
   * @param coord
   * @param color cella színe
   */
  public void setCellBackground(DCoord coord, Color color) {
    screenCells[coord.getRow()][coord.getCol()].setBackground(color);
  }

  /**
   * A képernyön beállítja az adott lehetöségérték kijelöltségi állapotát.
   *
   * @param state
   */
  public void setCandidateMark(DState state) {
    byte row = state.getRow();
    byte col = state.getCol();
    screenCells[row][col].setCandidateMark(state.getNumber(), state.getState());
  }

  /**
   * Be/kikapcsolja a jelöltet.
   *
   * @param state állapot
   */
  public void setCandidateState(DState state) {
    byte ms = state.getWhere();
    String candidates = getValidCandidates(new DWhere(ms, state.getCoord()));
    if ((state.getState() == S_ON) && (!isCandidateStateOn(state))) {
      String str2 = Basics.insertChar(candidates, "" + state.getNumber());
      candidates = (candidates.isEmpty() ? "" + state.getNumber() : str2);
      String values = "{" + candidates + "}";
      setCell(new DWhere(ms, state.getCoord()), values);
    }
    DNumber number = new DNumber(new DWhere(ms, state.getCoord()), state.getNumber());
    boolean criterion = isCandidateStateOn(number);
    if ((state.getState() == S_OFF) && (criterion)) {
      String values = getCell(new DWhere(ms, state.getCoord()));
      String cell = Basics.deleteChar(values, "" + state.getNumber());
      setCell(new DWhere(ms, state.getCoord()), cell);
    }
  }

  /**
   * Egy cella értékeinek beállítása.
   * 
   * @param where
   * @param values lehetőség-értékek, vagy megoldásérték
   */
  public void setCell(DWhere where, String values) {
    if (where.getWhere() == OP_SCREEN) {
      screenCells[where.getRow()][where.getCol()].setCell(values);
    } else {
      virtualCells[where.getRow()][where.getCol()] = values;
    }
  }

  /**
   * Cella jelölését kapcsolja ki/be.<br><br> Bekapcsolásnál pedig az alapján
   * szinez, hogy a cella RESULT vagy ASSIST.
   *
   * @param coord
   * @param markType C_RESULT / C_ASSIST
   * @param state S_ON / S_OFF
   */
  public void setCellMark(DCoord coord, boolean markType, boolean state) {
    if (state == S_OFF) {
      setCellBackground(coord, Color.WHITE);
    } else {
      if (markType == C_ASSIST) {
        setCellBackground(coord, parent.storeProp.getActualColor(COLOR_ASSIST_CELL));
      } else {
        setCellBackground(coord, parent.storeProp.getActualColor(COLOR_RESULT_CELL));
      }
    }
    screenCells[coord.getRow()][coord.getCol()].repaint();
  }

  /**
   * A memóriából áttölti a képernyöre a táblát, vagy viszont.
   *
   * @param toWhere hova (OP_SCREEN - OP_MEMORY)
   */
  public void showTable(byte toWhere) {
    byte dest = toWhere;
    byte sour = ((dest == OP_SCREEN) ? (OP_MEMORY) : (OP_SCREEN));
    resetTable(dest, S_OFF);
    DCoord pos;
    for (byte row = 0; row < 9; row++) {
      for (byte col = 0; col < 9; col++) {
        pos = new DCoord(row, col);
        setCell(new DWhere(dest, pos), getCell(new DWhere(sour, pos)));
      }
    }
    setEmptyCellsCounter(dest, recountEmptyCells(sour));
    parent.setupFrame();
  }

  /**
   * Indextábla feltöltése.
   */
  public void fillIndexTable() {
    DCoord coord = new DCoord();
    for (byte areaForm = 0; areaForm < 3; areaForm++) {
      for (byte area = 0; area < 9; area++) {
        for (byte position = 0; position < 9; position++) {
          workAreas[areaForm][area][position] = new DCoord((byte) 0, (byte) 0);
          switch (areaForm + 1) {
            case A_ROW:
              workAreas[areaForm][area][position].setCoord(area, position);
              break;
            case A_COL:
              workAreas[areaForm][area][position].setCoord(position, area);
              break;
            case A_BLOCK:
              coord.setRow((byte) (((area / 3) * 3) + (position / 3)));
              coord.setCol((byte) (((area % 3) * 3) + (position % 3)));
              workAreas[areaForm][area][position].setCoord(coord);
              break;
          } // switch
        } // for - position
      } // for - area
    } // for - areaForm
  }

  /**
   * Beállítja a cella eredményértékét, de nem aktualizálja az üres cellák számát
   * és a munkaterület celláit.
   *
   * @param number
   */  
  public void setGridResult(DNumber number) {
    if (number.getWhere() == OP_SCREEN) {
      screenCells[number.getRow()][number.getCol()].setResult(number.getNumber());
    } else {
      virtualCells[number.getRow()][number.getCol()] = "" + number.getNumber();
    }
  }

  /**
   * Az eredményértéket hozzáadja a result listához.
   * 
   * @param stepOrder
   * @param rule
   * @param number 
   */
  public void setListResult(short stepOrder, byte rule, DNumber number) {
    DWhere where = new DWhere(number.getWhere(), number.getCoord());
    parent.lists.addResult(stepOrder, rule, number, T_RESULT, "");
    for (byte cand = 1; cand <= 9; cand++) {
      DNumber number2 = new DNumber(where, cand);
      boolean cond1 = (isCandidateStateOn(number2) == S_ON);
      boolean cond2 = (cand != number.getNumber());
      if (cond1 && cond2) {
        parent.lists.addResult(stepOrder, (byte) 0, number, T_POSSIBILITY, "");
      }
    }
  }

// ----------------------------------------------------------------------------- Cellapozíciók kezelése

  /**
   * Visszaadja az aktuális cella X koordinátáját.
   * 
   * @return the actCellX
   */
  public DCoord getActCell() {
    return actCell;
  }

  /**
   * Visszaadja az előző cella Y koordinátáját.
   *
   * @return the lastCellY
   */
  public DCoord getPrevCell() {
    return prevCell;
  }

  /**
   * Beállítja az aktuális cella koordinátáit.
   *
   * @param coord
   */
  public void setActCell(DCoord coord) {
    this.actCell = coord;
  }

  /**
   * Beállítja az előző cella X koordinátáját.
   *
   * @param prevCell
   */
  public void setPrevCell(DCoord prevCell) {
    this.prevCell = prevCell;
  }

// ----------------------------------------------------------------------------- Számlálók kezelése

  /**
   * Visszaadja a memória/képernyő-rács üres celláinak a számát.
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @return
   */
  public short getEmptyCellsCounter(byte where) {
    return (where == OP_SCREEN ? emptyScreenCells : emptyVirtualCells);
  }

  /**
   * Visszaadja az adott rács nem megfejtett celláinak a számát.
   * 
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @return
   */
  public String getEmptyCellsCounterString(byte where) {
    return "" + getEmptyCellsCounter(where);
  }

  /**
   * Megoldatlan cellák számának beállítása adott értékre
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @param counter üres cellák száma
   */
  public void setEmptyCellsCounter(byte where, short counter) {
    if (where == OP_SCREEN) {
      emptyScreenCells = counter;
    } else {
      emptyVirtualCells = counter;
    }
  }

  /**
   * Növeli / csökkenti az üres cellák számlálót.
   * 
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @param adding a hozzá adandó érték. ha negatív a hozzáadandó érték, akkor
   * értelemszerüen csökenti a számlálót.
   */
  public void setEmptyCellsCounterByAdding(byte where, short adding) {
    setEmptyCellsCounter(where, (short) (getEmptyCellsCounter(where) + adding));
  }

// ----------------------------------------------------------------------------- Egyéb

  /**
   * Szerkesztő üzemmód lekérdezése
   *
   * @return üzemmód állapota
   */
  public boolean isEditable() {
    return editable;
  }

  /**
   * A szerkesztömód be/kikapcsolására szolgál.
   * 
   * @param state S_ON/S_OFF
   */
  public void setEditable(boolean state) {
    editable = state;
  }

}
