/*
 * Sudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.Basis.Basics;
import SudokuPIPkg.DataForms.DColors;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.Basis.IConst;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

/**
 * A globális változók tárolása, valamint a paraméterek beállítása a feladata.
 */
public class Props extends JDialog implements ActionListener, IConst {

  /**
   * A szülőablakra mutató változó
   */
  private Frame parent = null;

  /**
   * Az állapotok tárolója.
   */
  private Properties defaultProp, prevProp, prop;

  private JPanel p3Buttons, pSettings, pPages;
  private JButton bOk, bMegse, bReset;
  private JCheckBox[] cbRules;
  private JCheckBox cbRulesGetOut, cbRulesEncapsulate;
  private JLabel[] lElement, lElementStreak;
  private JTabbedPane tpSettings;
  private JRadioButton[] rbNSPosition = new JRadioButton[2],
                         rbSearchWay = new JRadioButton[2],
                         rbScheme = new JRadioButton[4],
                         rbOpMode = new JRadioButton[2];
  private JPanel[] pSett = new JPanel[3];
  private ButtonGroup bgColorScheme, bgOperatingMode, bgSearchWay, bgNSPosition;
  private ArrayList<JComponent> GUIElements = new ArrayList<>();
  private GridBagConstraints gbcCells = new GridBagConstraints();

  /**
   * Létrehozza az objektumot, majd beállítja az alapértelmezet állapotokat.
   * Végül ha létezik a SudokuPI.ini file, akkor beolvassa azt, ha pedig nem,
   * akkor felveszi az alapértelmezett értékeket.
   *
   * @param parent szülő ablakára mutató pointer
   * @param modal Az ablak modális e. true esetén nem tudjuk e ablak bezárása
   * nélkül a szülő ablakot aktiválni.
   */
  public Props(Frame parent, boolean modal) {
    super(parent, modal);
    this.parent = parent;
    init();
  }

  /**
   * Beállítja a kezdőértékeket.
   */
  private void init() {
    defaultProp = new Properties();
    prevProp = new Properties();
    prop = new Properties();
    setupDefaultProperties();    
    File fileIni = new File(INI_FILE_NAME);
    if (fileIni.exists()) {
      try {
        prop.load(new FileInputStream(fileIni));
      }
      catch (IOException e) {
        copyProperty(defaultProp, prop);
        e.printStackTrace();
      }
    } else {
      copyProperty(defaultProp, prop);
    }
    copyProperty(prop, prevProp);
    settingsGUI();
    resetElements(S_ON);
    resetDialog();
    setupPages();
  }

  /**
   * Meghatározza az alapértékeket.
   */
  private void setupDefaultProperties() {
    // Az alapértelmezett értékek beállítása.
    for (int c = 1; c < 11; c++) {
      defaultProp.setProperty(RULESNAMES[c], "1");
    }
    defaultProp.setProperty(RULESNAMES[6], "0");
    defaultProp.setProperty("RulesGetOut", "0");
    defaultProp.setProperty("Encapsulate", "0");
    defaultProp.setProperty("NakedSinglePos", "1");
    defaultProp.setProperty("SearchWay", "1");
    defaultProp.setProperty("ColorScheme", "1");
    defaultProp.setProperty("OperatingMode", "2");
  }

  /**
   * Megjeleníti a beállításokat végző ablakot.
   */
  public void showPropsFrame() {
    setTitle("SudokuPI - beállítások");
    setBounds(0, 0, 400, 400);
    setLocationRelativeTo(parent);
    setVisible(true);
  }

  /**
   * A beállításokat végző ablak grafikus felületének kialakítását végzi el.
   */
  public void settingsGUI() {
    addComponentListener(new ComponentAdapter() {
      @Override
      public void componentShown(ComponentEvent e) {
        Dimension size = new Dimension();
        size = rbScheme[1].getSize();
        for(JLabel element : lElement) {
          if (element.getSize().width > size.width) {
            size.width = element.getSize().width;
          }
        }
        for(JLabel element : lElement) {
          element.setSize(size);
          element.setPreferredSize(size);
          element.setMinimumSize(size);
        }
      }
    });
    pSettings = new JPanel(new BorderLayout());
    add(pSettings);
    pPages = new JPanel();
    p3Buttons = new JPanel();
    pSettings.add(BorderLayout.SOUTH, p3Buttons);
    pSettings.add(BorderLayout.CENTER, pPages);

    // alsó rész, ahol a gombok vannak
    p3Buttons.setBackground(Color.LIGHT_GRAY);
    bOk = new JButton("OK");
    bMegse = new JButton("Mégse");
    bReset = new JButton("Alap");
    addListener(bOk, "opOK", this);
    addListener(bMegse, "opMégse", this);
    addListener(bReset, "opReset", this);
    p3Buttons.add(bOk);
    p3Buttons.add(bMegse);
    p3Buttons.add(bReset);

    pages();
    page1();
    page2();
    page3();
  }

  /**
   * felső rész, ahol a lapok vannak
   */
  private void pages() {
    Border margin = new EmptyBorder(30, 30, 30, 30);
    pPages.setLayout((new BoxLayout(pPages, BoxLayout.Y_AXIS)));
    tpSettings = new JTabbedPane();
    pPages.add(tpSettings);
    for (int c = 0; c < 3; c++) {
      pSett[c] = new JPanel();
      pSett[c].setLayout(new GridBagLayout());
      pSett[c].setBorder(margin);
      tpSettings.addTab(SETTINGS_PAGES[c], pSett[c]);
    }
    bgColorScheme = new ButtonGroup();
    bgOperatingMode = new ButtonGroup();
    bgSearchWay = new ButtonGroup();
    bgNSPosition = new ButtonGroup();
    gbcCells.ipady = 2;
    gbcCells.anchor = GridBagConstraints.LINE_START;
  }

  /**
   * Használandó szabályok
   */
  private void page1() {
    cbRules = new JCheckBox[11];
    for (int c = 1; c < 11; c++) {
      cbRules[c] = new JCheckBox((Basics.padR("" + c, " ", 3)) + ". " + RULESSHOW[c]);
      cbRules[c].setFont(new Font("Consolas", Font.BOLD, 12));
      addGuiElement(P_UR, new DCoord((byte) (c - 1), (byte) 0), cbRules[c], "R_" + c, (c > 1));
    }
    cbRules[1].setEnabled(S_OFF); // Azért, mert a NakedSingle szabályt mindig használni kell.
    addGuiElement(P_UR, new DCoord((byte) 0, (byte) ++gbcCells.gridx), new JLabel(Basics.space(10)), "", S_OFF);
    cbRulesGetOut = new JCheckBox("Szabály kikerülés");
    addGuiElement(P_UR, new DCoord((byte) 0, (byte) ++gbcCells.gridx), cbRulesGetOut, "G_0", S_ON);
    cbRulesEncapsulate = new JCheckBox("Szabály betokozás");
    addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), cbRulesEncapsulate, "E_0", S_ON);
    addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), new JLabel(Basics.space(1)), "", S_OFF);
    addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), new JLabel("Naked Single helye:"), "", S_OFF);
    for (int c = 0; c < 2; c++) {
      rbNSPosition[c] = new JRadioButton(SP_NSPOS[c]);
      bgNSPosition.add(rbNSPosition[c]);
      addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), rbNSPosition[c], "N_" + (c + 1), S_ON);
    }
    addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), new JLabel(Basics.space(1)), "", S_OFF);
    addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), new JLabel("Haladási irány:"), "", S_OFF);
    for (int c = 0; c < 2; c++) {
      rbSearchWay[c] = new JRadioButton(SP_WAY[c]);
      bgSearchWay.add(rbSearchWay[c]);
      addGuiElement(P_UR, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), rbSearchWay[c], "W_" + (c + 1), S_ON);
    }
  }

  /**
   * Színsémát kiválasztó lap
   */
  private void page2() {
    rbScheme = new JRadioButton[4];
    lElement = new JLabel[ELEMENTS_COUNT];
    lElementStreak = new JLabel[ELEMENTS_COUNT];
    addGuiElement(P_CS, new DCoord((byte) 0, (byte) 0), new JLabel("Színsémák:"), "", S_OFF);
    addGuiElement(P_CS, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), new JLabel(Basics.space(1)), "", S_OFF);
    for (int c = 0; c < 4; c++) {
      rbScheme[c] = new JRadioButton((c + 1) + ". " + SCHEME_NAME[c]);
      bgColorScheme.add(rbScheme[c]);
      addGuiElement(P_CS, new DCoord((byte) ++gbcCells.gridy, (byte) gbcCells.gridx), rbScheme[c], "S_" + (c + 1), S_OFF);
    }
    addGuiElement(P_CS, new DCoord((byte) 0, (byte) ++gbcCells.gridx), new JLabel(Basics.space(4)), "", S_OFF);
    addGuiElement(P_CS, new DCoord((byte) 0, (byte) ++gbcCells.gridx), new JLabel(Basics.space(7) + "Szinek:"), "", S_OFF);
    for (int c = 0; c < ELEMENTS_COUNT; c++) {
      lElement[c] = new JLabel(Basics.space(8) + ELEMENTS[c]);
      lElementStreak[c] = new JLabel(Basics.space(16));
      lElementStreak[c].setOpaque(S_ON);
      addGuiElement(P_CS, new DCoord((byte) (c+2), (byte) 2), lElement[c], "", S_OFF);
      addGuiElement(P_CS, new DCoord((byte) (c+2), (byte) 3), lElementStreak[c], "", S_OFF);
    }
  }

  /**
   * Üzemmód kiválasztó lap.
   */
  private void page3() {
    addGuiElement(P_OS, new DCoord((byte) 0, (byte) 0), new JLabel("Üzemmód:"), "", S_OFF);
    for (byte c = 0; c < 2; c++) {
      rbOpMode[c] = new JRadioButton(OP_MODE[c]);
      bgOperatingMode.add(rbOpMode[c]);
      addGuiElement(P_OS, new DCoord((byte) c, (byte) 1), rbOpMode[c], "O_" + (c + 1), S_ON);
    }
  }

  /**
   * Elhelyez egy elemet az adott lap megfelelő pozíciójára.
   * 
   * @param page melyik lapra helyezze az elemet
   * @param coord a griden belüli koordináta
   * @param comp a kihelyezendő elem
   * @param comm az eseménykezelőhöz rendelt parancs-név
   * @param mustTurn azt tárolja, hogy az elem engedélyezési állapotát kapcsolgatni kell e
   */
  private void addGuiElement(int page, DCoord coord, JComponent comp, String comm, boolean mustTurn) {
    gbcCells.gridy = coord.getRow();
    gbcCells.gridx = coord.getCol();
    if ((!comm.isEmpty()) && (comp instanceof AbstractButton)) {
      addListener((AbstractButton) comp, comm, this);
    }
    pSett[page].add(comp, gbcCells);
    if (mustTurn) {
      GUIElements.add(comp);
    }
  }

  /**
   * Az AbstractButton ősű komponensekhez eseménykezelőt rendel.
   * 
   * @param ab az objektum neve
   * @param comm a parancs
   * @param al
   */
  public void addListener(AbstractButton ab, String comm, ActionListener al) {
    ab.setActionCommand(comm);
    ab.addActionListener(al);
  }

  /**
   * Beállítja, hogy az üzemmód lapon lévő üzenet láthatósági állapotát.
   */
  public void setupPages() {
    boolean isShowStarted = parent.myShow.isShowStarted();
    resetElements(!isShowStarted);
    tpSettings.setSelectedIndex((isShowStarted ? 1 : 0));
    showColors();
  }

  /**
   * Beállítja a kezelőfelületen lévő elemek állapotát a prop tárolóban lévő
   * értékek alapján.
   */
  private void resetDialog() {
    for (int c = 1; c < 11; c++) {
      cbRules[c].setSelected(getRuleStateFromProp(c));
    }
    cbRulesGetOut.setSelected(getRulesGetOutFromProp());
    cbRulesEncapsulate.setSelected(getEncapsulateFromProp());
    resetRadioButtons(rbNSPosition, getNakedSinglePosFromProp()-1);
    resetRadioButtons(rbSearchWay, getSearchWayFromProp()-1);
    resetRadioButtons(rbScheme, getColorSchemeFromProp()-1);
    resetRadioButtons(rbOpMode, getOperatingModeFromProp()-1);
    for (int c = 0; c < 3; c++) {
      pSett[c].repaint();
    }
  }

  /**
   * A radio-gombok beállítása a feladata.
   * Mindet ki, majd az itemIndex-es elemet be.
   * 
   * @param rbGroup
   * @param itemIndex 
   */
  private void resetRadioButtons(JRadioButton[] rbGroup, int itemIndex) {
    for (JRadioButton oneButton : rbGroup) {
      oneButton.setSelected(S_OFF);
    }
    if (itemIndex > -1) {
      rbGroup[itemIndex].setSelected(S_ON);
    }
  }

  /**
   * Összeszámlálja a használandó szabályokat.
   * 
   * @return 
   */
  public byte activeRulesCount() {
    byte count = 0;
    for (byte c = 1; c < 11; c++) {
      count += (isRuleUsable(c) ? 1 : 0);
    }
    return count;
  }

  /**
   * A beállítások dialógusablak eseménykezelője.
   * 
   * @param ae
   */
  @Override
  public void actionPerformed(ActionEvent ae) {
    String sender = ae.getActionCommand();
    File fileIni = new File(INI_FILE_NAME);
    if (sender.equals("opOK")) {
      if (fileIni.exists()) {
        JPanel pConfirmSave = new JPanel();
        pConfirmSave.setSize(50, 150);
        pConfirmSave.add(new JLabel("A " + INI_FILE_NAME + " file már létezik. Felülírható?"));
        int sel = JOptionPane.showOptionDialog(pSettings, pConfirmSave, "Mentés megerősítés ...",
                0, JOptionPane.PLAIN_MESSAGE, null, OPTIONS_IGENNEM, 0);
        sender = (sel == 0 ? "opReallySave" : "opMégse");
      } else {
        sender = "opReallySave";
      }
    }
    if (sender.equals("opMégse")) {
      copyProperty(prevProp, prop);
      resetDialog();
      closeDialog();
    }
    if (sender.equals("opReallySave")) {
      saveProps();
      parent.myShow.setupRulesChain();
      closeDialog();
    }
    if (sender.equals("opReset")) {
      copyProperty(defaultProp, prop);
      resetDialog();
    }
    if (sender.substring(0, 2).equals("R_")) {
      byte r = Byte.parseByte(sender.substring(2));
      setRuleStateToProp(r, getRuleStateFromDialog(r));
    }
    if (sender.substring(0, 2).equals("G_")) {
      setRulesGetOutToProp(getRulesGetOutFromDialog());
    }
    if (sender.substring(0, 2).equals("E_")) {
      setEncapsulateToProp(getEncapsulateFromDialog());
    }
    if (sender.substring(0, 2).equals("N_")) {
      setNakedSinglePosToProp(getNakedSinglePosFromDialog());
    }
    if (sender.substring(0, 2).equals("W_")) {
      setSearchWayToProp(getSearchWayFromDialog());
    }

    if (sender.substring(0, 2).equals("S_")) {
      setColorSchemeToProp("" + getColorSchemeFromDialog());
      showColors();
    }

    if (sender.substring(0, 2).equals("O_")) {
      setOperatingModeToProp(getOperatingModeFromDialog());
    }
  }

  /**
   * "Bezárja" a beállításokat végző ablakot
   */
  private void closeDialog() {
//    dispose();
    setVisible(S_OFF);
  }

  /**
   * Elmenti a beállított értékeket.
   */
  private void saveProps() {
    for (byte c = 1; c < 11; c++) {
      prop.setProperty(RULESNAMES[c], (cbRules[c].isSelected() ? "1" : "0"));
    }
    prop.setProperty("RulesGetOut", "" + getRulesGetOutFromDialog());
    prop.setProperty("Encapsulate", "" + getEncapsulateFromDialog());
    prop.setProperty("NakedSinglePos", "" + getNakedSinglePosFromDialog());
    prop.setProperty("SearchWay", "" + getSearchWayFromDialog());
    prop.setProperty("ColorScheme", "" + getColorSchemeFromDialog());
    prop.setProperty("OperatingMode", "" + getOperatingModeFromDialog());
    BufferedWriter fileOut = null;
    String newLine = System.getProperty("line.separator");
    try {
      File fileIni = new File(INI_FILE_NAME);
      if (!fileIni.exists()) {
        fileIni.createNewFile();
      }
      fileOut = new BufferedWriter(new FileWriter(fileIni));
      fileOut.write("; Saved settings of SudokuPI.JAR software." + newLine + newLine);
      fileOut.write("[Rules switches]" + newLine);
      fileOut.write("; 0=Off, 1=On" + newLine);
      for (byte c = 1; c < 11; c++) {
        fileOut.write(RULESNAMES[c] + "=" + prop.getProperty(RULESNAMES[c]) + newLine);
      }
      fileOut.write(newLine + "; Search mode options" + newLine);
      fileOut.write("RulesGetOut=" + prop.getProperty("RulesGetOut") + newLine);
      fileOut.write("Encapsulate=" + prop.getProperty("Encapsulate") + newLine);
      fileOut.write("NakedSinglePos=" + prop.getProperty("NakedSinglePos") + newLine);
      fileOut.write("SearchWay=" + prop.getProperty("SearchWay") + newLine + newLine);
      fileOut.write("[Others]" + newLine);
      fileOut.write("ColorScheme=" + prop.getProperty("ColorScheme") + newLine);
      fileOut.write("; Operating mode: 1=Screen, 2=Memory" + newLine);
      fileOut.write("OperatingMode=" + prop.getProperty("OperatingMode") + newLine);
      fileOut.flush();
    } catch (FileNotFoundException ex) {
      Logger.getLogger(getName()).log(Level.SEVERE, null, ex);
    } catch (IOException ex) {
      Logger.getLogger(getName()).log(Level.SEVERE, null, ex);
    } finally {
      if (fileOut != null) {
        try {
          fileOut.close();
        } catch (IOException ex) {
          Logger.getLogger(getName()).log(Level.SEVERE, null, ex);
        }
      }
    }
  }

// ----------------------------------------------------------------------------- Színek és sémák kezelése

  /**
   * Az előre beállított színséma alapján visszaadja az adott elemhez tartozó
   * színt.
   * 
   * @param what az adott elem.
   * @return
   */
  public Color getActualColor(byte what) {
    return (COLOR_SCHEME[getColorSchemeFromProp() - 1][what]);
  }

  /**
   * Megvizsgálja, hogy a séma megváltozott e.
   * 
   * @return
   */
  public boolean isColorSchemeChanged() {
    return ((getPrevColorScheme() - getColorSchemeFromDialog()) != 0);
  }

  /**
   * Az aktuális színséma sorszámát állítja be.
   * 
   * @param scheme színséma sorszáma, pl. SCHEME_NATURE.
   */
  private void setColorSchemeToProp(String scheme) {
    prop.setProperty("ColorScheme", scheme);
  }

  /**
   * Az aktuális színséma sorszámát kérdezi le.
   * 
   * @return 1..4 szám
   */
  public byte getColorSchemeFromProp() {
    return (Byte.parseByte(prop.getProperty("ColorScheme")));
  }
  
  /**
   * Kiolvassa a kiválasztott színsémát a dialógusablakból.
   * 
   * @return
   */
  private byte getColorSchemeFromDialog() {
    return (Byte.parseByte(bgColorScheme.getSelection().getActionCommand().substring(2, 3)));
  }

  /**
   * Visszaadja a mentett színsémát.
   * 
   * @return
   */
  private byte getPrevColorScheme() { // ennek nincs set párja, nehogy valaki elkezdje állítgatni.
    return (Byte.parseByte(prevProp.getProperty("ColorScheme")));
  }

  /**
   * Visszaadja az aktuális sémához tartozó szineket.
   * 
   * @return
   */
  public DColors getActualColors() {
    DColors colors = new DColors();
    for (byte c = 0; c < 6; c++) {
      colors.setColorElement(c, getActualColor(c));
    }
    return colors;
  }


  /**
   * Átszínezi a színsávokat.
   */
  private void showColors() {
    for (byte c = 0; c < 6; c++) {
      lElementStreak[c].setBackground(getActualColor(c));
    }
  }

// ----------------------------------------------------------------------------- Üzemmódok kezelése
  
  /**
   * Az aktuális üzemmód sorszámát állítja be a propertyben.
   * 
   * @param opMode az üzemmód sorszáma
   */
  private void setOperatingModeToProp(byte opMode) {
    prop.setProperty("OperatingMode", "" + opMode);
  }

  /**
   * Kiolvassa a műveletvégzés módját a propertyből.
   * 
   * @return
   */
  public byte getOperatingModeFromProp() {
    return (Byte.parseByte(prop.getProperty("OperatingMode")));
  }

  /**
   * Kiolvassa a kiválasztott üzemmódot a dialógusablakból.
   * 
   * @return 
   */
  private byte getOperatingModeFromDialog() {
    return Byte.parseByte(bgOperatingMode.getSelection().getActionCommand().substring(2, 3));
  }

  /**
   * Az adott szabály állapotát állítja be a propertyben.
   * 
   * @param rule
   * @param state 
   */
  private void setRuleStateToProp(byte rule, boolean state) {
    prop.setProperty(RULESNAMES[rule], (state ? "1" : "0"));
  }

  /**
   * Kiolvassa az adott szabály állapotát a propertyből.
   * 
   * @param rule
   * @return 
   */
  public boolean getRuleStateFromProp(int rule) {
    return prop.getProperty(RULESNAMES[rule]).equals("1");
  }

  /**
   * Kiolvassa a kiválasztott szabály állapotát a dialógusablakból.
   * 
   * @param rule
   * @return 
   */
  private boolean getRuleStateFromDialog(int rule) {
    return cbRules[rule].isSelected();
  }

  /**
   * Visszaadja, hogy az adott szabály bekapcsolt állapotban van e.
   * 
   * @param rule
   * @return 
   */
  public boolean isRuleUsable(int rule) {
    return getRuleStateFromProp(rule);
  }

  /**
   * Szabály kikerülési kapcsolóhoz tartozó változó
   * állapotának lekérdezése.
   * 
   * @return
   */
  public boolean getRulesGetOutFromProp() {
    return prop.getProperty("RulesGetOut").equals("1");
  }

  /**
   * A szabály kikerülési kapcsoló állapotának elmentése
   * a propertybe.
   * 
   * @param state 
   */
  private void setRulesGetOutToProp(boolean state) {
    prop.setProperty("RulesGetOut", (state ? "1" : "0"));
  }

  /**
   * A szabály kikerülési kapcsoló állapotának kiolvasása
   * a dialógus-ablakból.
   * 
   * @return 
   */
  private boolean getRulesGetOutFromDialog() {
    return cbRulesGetOut.isSelected();
  }

  /**
   * Szabály betokozási kapcsoló állapotának kiolvasása
   * a propertyből.
   * 
   * @return 
   */
  public boolean getEncapsulateFromProp() {
    return prop.getProperty("Encapsulate").equals("1");
  }

  /**
   * Szabály betokozási kapcsoló állapotának eltárolása
   * a propertybe.
   * 
   * @param state 
   */
  private void setEncapsulateToProp(boolean state) {
    prop.setProperty("Encapsulate", (state ? "1" : "0"));
  }

  /**
   * Szabály betokozási kapcsoló állapotának kiolvasása
   * a dialógusablakból.
   * 
   * @return 
   */
  private boolean getEncapsulateFromDialog() {
    return cbRulesEncapsulate.isSelected();
  }

  /**
   * NakedSingle pozíció kiolvasása a propertyből.
   * 
   * @return 
   */
  public byte getNakedSinglePosFromProp() {
    return Byte.parseByte(prop.getProperty("NakedSinglePos"));
  }

  /**
   * NakedSingle pozíció eltárolása a propertybe.
   * 
   * @param pos 
   */
  private void setNakedSinglePosToProp(int pos) {
    prop.setProperty("NakedSinglePos", "" + pos);
  }

  /**
   * NakedSingle pozíció kiolvasása a dialógusablakból.
   * 
   * @return 
   */
  private byte getNakedSinglePosFromDialog() {
    return Byte.parseByte(bgNSPosition.getSelection().getActionCommand().substring(2, 3));
  }

  /**
   * Keresési irány kapcsoló kiolvasása a propertyből.
   * 
   * @return 
   */
  public byte getSearchWayFromProp() {
    return Byte.parseByte(prop.getProperty("SearchWay"));
  }

  /**
   * Keresési irány kapcsoló eltárolása a propertybe.
   * 
   * @param way 
   */
  private void setSearchWayToProp(int way) {
    prop.setProperty("SearchWay", "" + way);
  }

  /**
   * Keresési irány kapcsoló kiolvasása a dialógusablakból.
   * 
   * @return 
   */
  private byte getSearchWayFromDialog() {
    return Byte.parseByte(bgSearchWay.getSelection().getActionCommand().substring(2, 3));
  }

  /**
   * Beállítja a dialógus elemeinek engedélyezési státuszát.
   * Arra használatos, ha tudom, hogy elindult a bemutató,
   * akkor kikapcsolok minden dialógus-elemet a színkezelés
   * kivételével azért, hogy a bemutató közben ne lehessen
   * átállítani, mivel az befolyásolná a bemutató menetét.
   * 
   * @param way 
   */
  private void resetElements(boolean way) {
    GUIElements.stream().forEach((element) -> element.setEnabled(way));
  }

// ----------------------------------------------------------------------------- Egyéb

  /**
   * A filekiterjesztést adja vissza
   * 
   * @param needDot a pont is szerepeljen e a filekiterjesztés előtt
   * @return
   */
  public String getExtention(boolean needDot) {
    return (((needDot) ? "." : "") + EXTENSION);
  }

  /**
   * Átmásolja az adott property tömböt.
   * 
   * @param sour
   * @param dest 
   */
  private void copyProperty(Properties sour, Properties dest) {
    sour.entrySet().stream().forEach((entry) -> {
      dest.setProperty((String) entry.getKey(), (String) entry.getValue());
    });
  }

}
