/*
 * Szudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.Basis.Basics;
import SudokuPIPkg.DataForms.DWhere;
import SudokuPIPkg.DataForms.DAPos;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.DataForms.DNumber;
import SudokuPIPkg.DataForms.DArea;
import SudokuPIPkg.Basis.IConst;

public class Rules implements IConst{

  private Frame parent = null;

  public Rules(Frame parent) {
    super();
    this.parent = parent;
  }

  /**
   * Tiszta szabályok megoldófüggvénye
   * 
   * @param pWhere
   * @param ruleOrder a szabály neve
   * @return 
   */
  public boolean naked(byte pWhere, byte ruleOrder) {
    byte extent = getExtent(ruleOrder);
    short stepOrder = (short) (parent.lists.getStepCount() + 1);
    DAPos dAPos;
    DCoord coord;
    DWhere where;
    DNumber number;
    String[] cell = new String[5];
    byte[] resPosition = new byte[5];
    String resCand, resultPositions;
    byte areaForm, area, resIndex, areaCell;
    boolean isResulted = false;
    areaForm = 0;
    while ((areaForm < 3) && (!isResulted)) {
      area = 0;
      while ((area < 9) && (!isResulted)) {
        for (byte c = 0; c < 5; c++) {
          cell[c] = "";
          resPosition[c] = -1;
        }
        areaCell = 0;
        resIndex = 0;
        while ((resIndex < extent) && (areaCell < 9) && (!isResulted)) {
          dAPos = new DAPos(pWhere, areaForm, area, areaCell);
          coord = parent.pGrid.getCoordFromPosition(dAPos);
          where = new DWhere(pWhere, coord);
          if (!parent.pGrid.isCellResolved(where)) {
            cell[resIndex] = parent.pGrid.getValidCandidates(where);
            resPosition[resIndex] = areaCell;
            resIndex++;
            resCand = Basics.unionSets(cell[0], cell[1], cell[2], cell[3], cell[4]);
            if (resCand.length() > extent) {
              if (resIndex > 0) {
                cell[resIndex - 1] = (cell[resIndex - 1] + 1);
              }
              cell[resIndex] = "";
              resIndex--;
            }
            if (resIndex == extent) {
              resultPositions = "";
              for (byte iRPos = 0; iRPos < extent; iRPos++) {
                resultPositions += resPosition[iRPos];
              }
              for (byte iRPos = 0; iRPos < extent; iRPos++) {
                byte oneCand = Byte.parseByte(resCand.substring(iRPos, iRPos + 1));
                for (byte position = 0; position < 9; position++) {
                  String sPos = "" + position;
                  if (!resultPositions.contains(sPos)) {
                    dAPos = new DAPos(pWhere, areaForm, area, position);
                    where = new DWhere(pWhere, parent.pGrid.getCoordFromPosition(dAPos));
                    number = new DNumber(where, oneCand);
                    boolean cond1 = (parent.pGrid.isCandidateStateOn(number));
                    if (cond1) {
                      isResulted = true;
                      parent.lists.addResult(stepOrder, ruleOrder, number, T_POSSIBILITY, "");
                    }
                  }
                }
              }
              if (isResulted) {
                for (byte cell2 = 0; cell2 < 9; cell2++) {
                  dAPos = new DAPos(pWhere, areaForm, area, cell2);
                  coord = parent.pGrid.getCoordFromPosition(dAPos);
                  parent.lists.addAssist(stepOrder, coord, resultPositions.contains("" + cell2));
                }
              }
            }
          }
          areaCell++;
        }
        area++;
      }
      areaForm++;
    }
    return (isResulted);
  }

  /**
   * Rejtett szabályok megoldófüggvénye
   * 
   * @param pWhere
   * @param ruleOrder a szabály sorszáma
   * @return
   */
  public boolean hidden(byte pWhere, byte ruleOrder) {
    byte extent = getExtent(ruleOrder);
    short stepOrder = (short) (parent.lists.getStepCount() + 1);
    DArea dArea;
    DAPos dAPos;
    DCoord coord;
    DNumber number;
    String[] resPos = new String[5];
    String allPos;
    byte areaForm, area, solNumber;
    boolean isResulted = false;
    areaForm = 0;
    while ((areaForm < 1) && (!isResulted)) {
      area = 0;
      while ((area < 1) && (!isResulted)) {
        String resCand = "";
        for (byte c = 0; c < 5; c++) {
          resPos[c] = "";
        }
        byte cand = 1;
        solNumber = 0;
        while ((cand < 10) && (!isResulted)) {
          dArea = new DArea(pWhere, (byte) (areaForm + 1), area);
          String positions = parent.pGrid.getCandidatePositionsInArea(dArea, cand);
          if ((!positions.isEmpty()) && (positions.length() <= extent)) {
            allPos = Basics.unionSets(positions, resPos[0], resPos[1], resPos[2], resPos[3], resPos[4]);
            if (allPos.length() <= extent) {
              resPos[solNumber] = positions;
              resCand += "" + cand;
              solNumber++;
            }
            if (resCand.length() == extent) {
              allPos = Basics.unionSets(resPos[0], resPos[1], resPos[2], resPos[3], resPos[4]);
              for (byte position = 0; position < extent; position++) {
                byte onePos = Byte.parseByte(allPos.substring(position, position + 1));
                for (byte oneCand = 1; oneCand < 10; oneCand++) {
                  dAPos = new DAPos(pWhere, areaForm, area, onePos);
                  coord = new DCoord(parent.pGrid.getCoordFromPosition(dAPos));
                  boolean cond1 = (!resCand.contains("" + oneCand));
                  number = new DNumber(new DWhere(pWhere, coord), oneCand);
                  if (cond1 && (parent.pGrid.isCandidateStateOn(number))) {
                    isResulted = true;
                    parent.lists.addResult(stepOrder, ruleOrder, number, T_POSSIBILITY, "");
                  }
                } // for oneCand
              } // for position
              if (isResulted) {
                for (byte i = 0; i < 9; i++) {
                  dAPos = new DAPos(pWhere, areaForm, area, i);
                  coord = new DCoord(parent.pGrid.getCoordFromPosition(dAPos));
                  boolean type = (allPos.contains("" + i) ? C_RESULT : C_ASSIST);
                  parent.lists.addAssist(stepOrder, coord, type);
                }
              }
            } // if (resCand ...
          } // if (!positions.isEmpty() ...)
          cand++;
        } // while cand
        area++;
      } // while area
      areaForm++;
    } // while areaForm
    return (isResulted);
  }

  /**
   * Visszaadja, hogy az adott szabály esetében hány értékig keressen.
   * 
   * @param ruleOrder
   * @return 
   */
  private byte getExtent(byte ruleOrder) {
    byte extent = 0;
    extent = ((ruleOrder == 1) || (ruleOrder ==  2) ? 1 : extent);
    extent = ((ruleOrder == 4) || (ruleOrder ==  5) ? 2 : extent);
    extent = ((ruleOrder == 7) || (ruleOrder ==  8) ? 3 : extent);
    extent = ((ruleOrder == 9) || (ruleOrder == 10) ? 4 : extent);
    return extent;
  }

  /**
   * Naked Single szabály
   * Van e olyan cella, amiben csak egy lehetséges érték van, akkor ez lesz a
   * megoldás.
   *
   * @param pWhere
   * @return
   */
  public boolean nakedSingle(byte pWhere) {
    short stepOrder = (short) (parent.lists.getStepCount() + 1);
    byte row, col, candidate;
    boolean isResulted = false;
    DCoord coord;
    DWhere where;
    row = 0;
    while ((row < 9) && (!isResulted)) {
      col = 0;
      while ((col < 9) && (!isResulted)) {
        coord = new DCoord(row, col);
        where = new DWhere(pWhere, coord);
        if (parent.pGrid.getCandidatesCounterInCell(where) == 1) {
          isResulted = true;
          candidate = Byte.parseByte(parent.pGrid.getValidCandidates(where));
          parent.pGrid.setListResult((short) stepOrder, (byte) 1, new DNumber(where, candidate));
          parent.lists.addAssist((short) stepOrder, coord, C_RESULT);
          parent.myShow.rule_Cleaning(stepOrder, new DNumber(where, candidate));
        } // if
        col++;
      } // while - col
      row++;
    } // while - row
    return (isResulted);
  }

  /**
   * Intersection szabály,
   * Sor illetve oszlop szegmens és blokk viszonyait vizsgálja.
   *
   * @param pWhere
   * @return
   */
  public boolean interSection(byte pWhere) {
    short stepOrder = (short) (parent.lists.getStepCount() + 1);
    byte boY, boX, offsetY, offsetX, segment, candidate, count, c;
    byte blockNumber = 0, blockNumber2 = 0, col = 0, col2 = 0, row = 0, row2 = 0;
    boolean isResulted = false;
    boolean cond1, cond2, cond3;
    DArea area1, area2;
    DCoord coord;
    DNumber number;
    while ((blockNumber < 9) && (!isResulted)) {
      boY = (byte) ((blockNumber / 3) * 3);
      boX = (byte) ((blockNumber % 3) * 3);
      candidate = 1;
      while ((candidate <= 9) && (!isResulted)) {
        // rowly
        offsetY = 0;
        while ((offsetY < 3) && (!isResulted)) {
          count = 0;
          for (col = 0; col < 3; col++) {
            coord = new DCoord((byte) (boY + offsetY), (byte) (boX + col));
            number = new DNumber(new DWhere(pWhere, coord), candidate);
            if (parent.pGrid.isCandidateStateOn(number)) {
              count++;
            }
          }
          area1 = new DArea(pWhere, A_BLOCK, blockNumber);
          area2 = new DArea(pWhere, A_ROW, offsetY);
          cond1 = (count == parent.pGrid.getCountCandidateInArea(area1, candidate));
          cond2 = (count < parent.pGrid.getCountCandidateInArea(area2, candidate));
          if ((count > 1) && cond1 && cond2) {
            for (c = 0; c < 9; c++) {
              coord = new DCoord((byte) (boY + offsetY), c);
              cond3 = (!Basics.isPositionInBlock(coord, blockNumber));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              if (cond3 && (parent.pGrid.isCandidateStateOn(number))) {
                isResulted = true;
                number = new DNumber(new DWhere(pWhere, coord), candidate);
                parent.lists.addResult(stepOrder, (byte) 3, number, T_POSSIBILITY, "1");
                blockNumber2 = Basics.getBlocknumberFromPosition(coord);
              }
            }
          }
          if (isResulted) {
            boY = (byte) ((blockNumber2 / 3) * 3);
            boX = (byte) ((blockNumber2 % 3) * 3);
            for (byte offset = 0; offset < 9; offset++) {
              coord = new DCoord((byte) (boY + (offset / 3)), (byte) (boX + (offset % 3)));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond3 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if (((offset / 3) != offsetY) && cond3) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
            for (col2 = 0; col2 < 9; col2++) {
              coord = new DCoord(row, col2);
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond3 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if ((Basics.getBlocknumberFromPosition(coord) != blockNumber2) && cond3) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
          }
          offsetY++;
        }
        // colly
        offsetX = 0;
        while ((offsetX < 3) && (!isResulted)) {
          count = 0;
          for (row = 0; row < 3; row++) {
            coord = new DCoord((byte) (boY + row), (byte) (boX + offsetX));
            number = new DNumber(new DWhere(pWhere, coord), candidate);
            if (parent.pGrid.isCandidateStateOn(number)) {
              count++;
            }
          }
          area1 = new DArea(pWhere, A_BLOCK, blockNumber);
          area2 = new DArea(pWhere, A_COL, offsetX);
          cond1 = (count == parent.pGrid.getCountCandidateInArea(area1, candidate));
          cond2 = (count < parent.pGrid.getCountCandidateInArea(area2, candidate));
          if ((count > 1) && cond1 && cond2) {
            col = (byte) (boX + offsetX);
            for (c = 0; c < 9; c++) {
              coord = new DCoord(c, col);
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond3 = (parent.pGrid.isCandidateStateOn(number));
              if ((!Basics.isPositionInBlock(coord, blockNumber)) && cond3) {
                isResulted = true;
                number = new DNumber(new DWhere(OP_MEMORY, coord), candidate);
                parent.lists.addResult(stepOrder, (byte) 3, number, T_POSSIBILITY, "2");
                blockNumber2 = Basics.getBlocknumberFromPosition(coord);
              }
            }
          }
          if (isResulted) {
            boY = (byte) ((blockNumber2 / 3) * 3);
            boX = (byte) ((blockNumber2 % 3) * 3);
            for (byte offset = 0; offset < 9; offset++) {
              coord = new DCoord((byte) (boY + (offset / 3)), (byte) (boX + (offset % 3)));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond1 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if (cond1 && ((offset % 3) != offsetX)) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
            for (row2 = 0; row2 < 9; row2++) {
              coord = new DCoord(row2, col);
              cond1 = (Basics.getBlocknumberFromPosition(coord) == blockNumber);
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond2 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if (cond1 && cond2) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
          }
          offsetX++;
        }
        candidate++;
      }
      blockNumber++;
    } // while
    //                                  Case 2.
    row = 0;
    while ((row < 9) && (!isResulted)) {
      candidate = 1;
      while ((candidate <= 9) && (!isResulted)) {
        segment = 0;
        while ((segment < 3) && (!isResulted)) {
          count = 0;
          for (offsetX = 0; offsetX < 3; offsetX++) {
            coord = new DCoord(row, (byte) ((segment * 3) + offsetX));
            number = new DNumber(new DWhere(pWhere, coord), candidate);
            if (parent.pGrid.isCandidateStateOn(number)) {
              count++;
            }
          }
          coord = new DCoord(row, (byte) (segment * 3));
          area1 = new DArea(pWhere, A_ROW, row);
          area2 = new DArea(pWhere, A_BLOCK, Basics.getBlocknumberFromPosition(coord));
          cond1 = (count == parent.pGrid.getCountCandidateInArea(area1, candidate));
          cond2 = (count < parent.pGrid.getCountCandidateInArea(area2, candidate));
          if ((count > 1) && cond1 && cond2) {
            boY = (byte) ((row / 3) * 3);
            boX = (byte) ((segment * 3));
            offsetY = 0;
            while ((offsetY < 3) && (!isResulted)) {
              if ((boY + offsetY) != row) {
                for (offsetX = 0; offsetX < 3; offsetX++) {
                  coord = new DCoord((byte) (boY + offsetY), (byte) (boX + offsetX));
                  number = new DNumber(new DWhere(pWhere, coord), candidate);
                  if (parent.pGrid.isCandidateStateOn(number)) {
                    isResulted = true;
                    number = new DNumber(new DWhere(OP_MEMORY, coord), candidate);
                    parent.lists.addResult(stepOrder, (byte) 3, number, T_POSSIBILITY, "3");
                    blockNumber2 = Basics.getBlocknumberFromPosition(coord);
                  }
                }
              }
              offsetY++;
            }
          }
          if (isResulted) {
            boY = (byte) ((blockNumber2 / 3) * 3);
            boX = (byte) ((blockNumber2 % 3) * 3);
            offsetY = (byte) (row2 - boY);
            for (byte offset = 0; offset < 9; offset++) {
              coord = new DCoord((byte) (boY + (offset / 3)), (byte) (boX + (offset % 3)));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond3 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if (((offset / 3) != offsetY) && cond3) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
            for (col = 0; col < 9; col++) {
              coord = new DCoord(row2, col);
              cond3 = (blockNumber2 != Basics.getBlocknumberFromPosition(coord));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              if (cond3 && (parent.pGrid.isCandidateStateOn(number) == S_ON)) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
          }
          segment++;
        }
        candidate++;
      }
      row++;
    }
    col = 0;
    while ((col < 9) && (!isResulted)) {
      candidate = 1;
      while ((candidate <= 9) && (!isResulted)) {
        segment = 0;
        while ((segment < 3) && (!isResulted)) {
          count = 0;
          for (offsetY = 0; offsetY < 3; offsetY++) {
            coord = new DCoord( (byte) ((segment * 3) + offsetY), col);
            number = new DNumber(new DWhere(pWhere, coord), candidate);
            if (parent.pGrid.isCandidateStateOn(number)) {
              count++;
            }
          }
          area1 = new DArea(pWhere, A_COL, col);
          coord = new DCoord((byte) (segment * 3), col);
          area2 = new DArea(pWhere, A_BLOCK, Basics.getBlocknumberFromPosition(coord));
          cond1 = (count == parent.pGrid.getCountCandidateInArea(area1, candidate));
          cond2 = (count < parent.pGrid.getCountCandidateInArea(area2, candidate));
          if ((count > 1) && cond1 && cond2) {
            boY = (byte) (segment * 3);
            boX = (byte) ((col / 3) * 3);
            for (offsetX = 0; offsetX < 3; offsetX++) {
              if ((boX + offsetX) != col) {
                for (offsetY = 0; offsetY < 3; offsetY++) {
                  coord = new DCoord((byte) (boY + offsetY), (byte) (boX + offsetX));
                  number = new DNumber(new DWhere(pWhere, coord), candidate);
                  if (parent.pGrid.isCandidateStateOn(number)) {
                    isResulted = true;
                    number = new DNumber(new DWhere(OP_MEMORY, coord), candidate);
                    parent.lists.addResult(stepOrder, (byte) 3, number, T_POSSIBILITY, "4");
                    blockNumber2 = Basics.getBlocknumberFromPosition(coord);
                  }
                }
              }
            }
          }
          if (isResulted) {
            boY = (byte) ((blockNumber2 / 3) * 3);
            boX = (byte) ((blockNumber2 % 3) * 3);
            offsetX = (byte) (col2 - boX);
            for (byte offset = 0; offset < 9; offset++) {
              coord = new DCoord((byte) (boY + (offset / 3)), (byte) (boX + (offset % 3)));
              number = new DNumber(new DWhere(pWhere, coord), candidate);
              cond1 = (parent.pGrid.isCandidateStateOn(number) == S_ON);
              if (cond1 && ((offset % 3) != offsetX)) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
            for (row = 0; row < 9; row++) {
              coord = new DCoord(row, col2);
              cond1 = (blockNumber2 != Basics.getBlocknumberFromPosition(coord));
              number = new DNumber(new DWhere(OP_MEMORY, coord), candidate);
              if (cond1 && (parent.pGrid.isCandidateStateOn(number) == S_ON)) {
                parent.lists.addAssist(stepOrder, coord, C_ASSIST);
              }
            }
          }
          segment++;
        }
        candidate++;
      }
      col++;
    }
    return (isResulted);
  }

  /**
   * X-Wings szabály
   *
   * @param pWhere
   * @return
   */
  public boolean xWings(byte pWhere) {
    short stepOrder = (short) (parent.lists.getStepCount() + 1);
    byte row, row1, row2, col, col1, col2, candidate;
    DCoord coord;
    DNumber number;
    boolean cond1, cond2, isResulted = false;
    DArea area;
    // sor
    row1 = 0;
    while ((row1 < 8) && (!isResulted)) {
      candidate = 1;
      while ((candidate < 10) && (!isResulted)) {
        area = new DArea(pWhere, A_ROW, row1);
        if ((parent.pGrid.getCountCandidateInArea(area, candidate) == 2)) {
          col1 = parent.pGrid.getPositionInArea(area, (byte) 1, candidate);
          col2 = parent.pGrid.getPositionInArea(area, (byte) 2, candidate);
          row2 = (byte) (row1 + 1);
          while ((row2 < 9) && (!isResulted)) {
            area = new DArea(pWhere, A_ROW, row2);
            if ((parent.pGrid.getCountCandidateInArea(area, candidate) == 2)) {
              cond1 = (parent.pGrid.getPositionInArea(area, (byte) 1, candidate) == col1);
              cond2 = (parent.pGrid.getPositionInArea(area, (byte) 2, candidate) == col2);
              if (cond1 && cond2) {
                for (row = 0; row < 9; row++) {
                  if ((row != row1) && (row != row2)) {
                    coord = new DCoord(row, col1);
                    number = new DNumber(new DWhere(pWhere, coord), candidate);
                    if (parent.pGrid.isCandidateStateOn(number)) {
                      isResulted = true;
                      parent.lists.addResult(stepOrder, (byte) 6, number, T_POSSIBILITY, "1");
                    }
                    coord = new DCoord(row, col2);
                    number = new DNumber(new DWhere(pWhere, coord), candidate);
                    if (parent.pGrid.isCandidateStateOn(number)) {
                      isResulted = true;
                      parent.lists.addResult(stepOrder, (byte) 6, number, T_POSSIBILITY, "1");
                    }
                  }
                }
              }
            }
            row2++;
          }
        }
        candidate++;
      }
      row1++;
    }
    // oszlop
    col1 = 0;
    while ((col1 < 8) && (!isResulted)) {
      candidate = 1;
      while ((candidate < 10) && (!isResulted)) {
        area = new DArea(pWhere, A_COL, col1);
        if ((parent.pGrid.getCountCandidateInArea(area, candidate) == 2)) {
          row1 = parent.pGrid.getPositionInArea(area, (byte) 1, candidate);
          row2 = parent.pGrid.getPositionInArea(area, (byte) 2, candidate);
          col2 = (byte) (col1 + 1);
          while ((col2 < 9) && (!isResulted)) {
            area = new DArea(pWhere, A_COL, col2);
            if ((parent.pGrid.getCountCandidateInArea(area, candidate) == 2)) {
              cond1 = (parent.pGrid.getPositionInArea(area, (byte) 1, candidate) == row1);
              cond2 = (parent.pGrid.getPositionInArea(area, (byte) 2, candidate) == row2);
              if (cond1 && cond2) {
                for (col = 0; col < 9; col++) {
                  if ((col != col1) && (col != col2)) {
                    coord = new DCoord(row1, col);
                    number = new DNumber(new DWhere(pWhere, coord), candidate);
                    if (parent.pGrid.isCandidateStateOn(number)) {
                      isResulted = true;
                      parent.lists.addResult(stepOrder, (byte) 6, number, T_POSSIBILITY, "2");
                    }
                    coord = new DCoord(row2, col);
                    number = new DNumber(new DWhere(pWhere, coord), candidate);
                    if (parent.pGrid.isCandidateStateOn(number)) {
                      isResulted = true;
                      parent.lists.addResult(stepOrder, (byte) 6, number, T_POSSIBILITY, "2");
                    }
                  }
                }
              }
            }
            row2++;
          }
        }
        candidate++;
      }
      row1++;
    }
    return (isResulted);
  }

}
