/*
 * Szudoku program, amely a GDF részére diplomamunkámul szolgál
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.Basis.Basics;
import SudokuPIPkg.Basis.IConst;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.DataForms.DNumber;
import SudokuPIPkg.DataForms.DState;
import SudokuPIPkg.DataForms.DWhere;
import SudokuPIPkg.SudokuPI;
import javax.swing.JOptionPane;

/**
 * Bemutató-vezérlő osztály
 */
public class Show implements IConst {

  private Frame parent = null;

  /**
   * Elindult e a bemutató.
   */
  private boolean showStarted = ANS_NO;

  /**
   * Szabályláncok.
   */
  private String actRulesChain, baseRulesChain;

  /**
   * Az osztály konstruktora.
   * 
   * @param parent 
   */
  public Show(Frame parent) {
    super();
    this.parent = parent;
  }

  /**
   * A bejelölendő cellák jelöltségi állapotának beállítása vagy frissítése.
   * Ezt hívom meg átszinezéskor is, és S_ON állapottal.
   * 
   * @param state
   */
  public void assistsSetShow(boolean state) {
    short posFrom = parent.lists.getAssistPosition((short) ((parent.lists.getStepOrder())-1));
    short posTo = parent.lists.getAssistPosition(parent.lists.getStepOrder());
    for (short position = posFrom; position < posTo; position++) {
      DCoord coord = new DCoord(parent.lists.getAssistCoord(position));
      parent.pGrid.setCellMark(coord, parent.lists.isAssistTypeResult(position), state);
    }
  }

  /**
   * A resultList aktuális lépésének tartalmát jeleníti meg a táblán.
   * 
   * @param res Milyen állapotra állítsa be az eredmény-értéket.
   * @param mark Bejelölje-e az értéket.
   * @param doIt Hajtsa-e végre.
   */
  // Itt ha a res=W_FW, és az üzemmód=SCREEN, akkor végre is kell hajtania a lépést. 
  public void resultsSetShow(boolean res, boolean mark, boolean doIt) {
    DCoord coord;
    DNumber number;
    DWhere where;
    DState state;
    byte value, rule;
    boolean cond1, cond2, resultType, isFirstLine = true;
    String comment;
    short stepOrder = parent.lists.getStepOrder();
    short posFrom = parent.lists.getResultPosition((short) ((stepOrder)-1));
    short posTo = parent.lists.getResultPosition(stepOrder);
    for (short position = posFrom; position < posTo; position++) {
      coord = parent.lists.getResultCoord(position);
      value = parent.lists.getResultValue(position);
      resultType = parent.lists.isResultTypeResult(position);
      where = new DWhere(OP_SCREEN, coord);
      number = new DNumber(where, value);
      if (doIt) {
        if (resultType == T_RESULT) {
          if (res == S_ON) {
            parent.pGrid.setGridResult(number);
            if (parent.pGrid.isEditable()) {
              parent.pGrid.changeCell(coord);
              parent.setupToolbarNumButtons(parent.pGrid.getValidCandidates(new DWhere(OP_SCREEN, coord)));
            }
          } else {
            byte cand = value;            
            parent.pGrid.clearResult(new DWhere(OP_SCREEN, coord));
            for(byte c = 1; c < 10; c++) {
              if (c != cand) {
                state = new DState(new DNumber(where, c), S_OFF);
                parent.pGrid.setCandidateState(state);
              }
            }
          }
          parent.pGrid.setEmptyCellsCounterByAdding(OP_SCREEN, (short) (res == S_OFF ? 1 : -1));
        } else {
          parent.pGrid.setCandidateState(new DState(number, !res));
        }
      } else {  // doIt
        if (resultType != T_RESULT) {
          parent.pGrid.setCandidateMark(new DState(number, mark));
        }
      }
      if ((mark) && (isFirstLine)) {
        rule = parent.lists.getResultRuleOrder(position);
        comment = parent.lists.getResultShowComment(position);
        String resultText = stepOrder + ". " + RULESSHOW[rule] + comment;
        parent.showResultText(OP_SCREEN, resultText);
        parent.showHitsToolTipText(rule);
        isFirstLine = !isFirstLine;
      }
    }
    if ((parent.myShow.isShowStarted()) && (doIt)) {
      cond1 = (stepOrder > 0);
      cond2 = (parent.storeProp.getOperatingModeFromProp() == OP_MEMORY);
      parent.setToolbarButtonEnabled(B_BACK, ((cond1 && cond2) ? S_ON : S_OFF));
      cond1 = (stepOrder > parent.lists.getStepCount());
      parent.setToolbarButtonEnabled(B_FORWARD, ((cond1) ? S_OFF : S_ON));
    }
    parent.showEmptyCells(OP_SCREEN, parent.pGrid.getEmptyCellsCounterString(OP_SCREEN));
    parent.pGrid.repaint();
  }

  /**
   * Lépés visszafelé.
   */
  public void stepBack() {
    assistsSetShow(S_OFF);
    resultsSetShow(W_BK, S_OFF, D_DONTDO);
    parent.lists.decStepOrder();
    assistsSetShow(S_ON);
    resultsSetShow(W_BK, S_ON, D_DO);
    boolean cond1 = (parent.storeProp.getOperatingModeFromProp() == OP_MEMORY);
    boolean cond2 = (parent.lists.getStepOrder() > 1);
    parent.setToolbarButtonEnabled(B_BACK, (cond1 && cond2));
  }

  /**
   * Lépés előre
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   */
  public void stepForward(byte where) {
    if (where == OP_SCREEN) {
      stepForwardScreen();
    } else {
      stepForwardMemory();
    }
  }

  /**
   * Lépés előre képernyőn.
   */
  private void stepForwardScreen() {
    boolean cond1;
    if (parent.lists.getStepCount() > 0) { // Volt előző lépés
      assistsSetShow(S_OFF);
      resultsSetShow(W_FW, S_OFF, D_DO);
      parent.lists.clearAssistAndResultLists();
    }   
    if (!parent.pGrid.isTableResolved(OP_SCREEN)) {
      baseRulesChain = setupRulesChain();
      setActRulesChain(getBaseRulesChain());
      if (oneStep(OP_SCREEN, (byte) 0) > 0) { // talált következő lépést
        assistsSetShow(S_ON);
        resultsSetShow(W_FW, S_ON, D_DONTDO);
      } else {
        parent.setToolbarButtonEnabled(B_FORWARD, S_OFF);
        cond1 = parent.pGrid.isTableResolved(OP_SCREEN);
        String message = (cond1 ? M_PUZZLE_RESOLVED : M_PUZZLE_NOT_RESOLVED);
        JOptionPane.showMessageDialog(null, message);
      }
    } else { // Ha meg van fejtve a tábla
      parent.setToolbarButtonEnabled(B_FORWARD, S_OFF);
      JOptionPane.showMessageDialog(SudokuPI.getpSudoku(), M_PUZZLE_RESOLVED);
    }
  }

  /**
   * Lépés előre a memóriában.
   * <br><br>
   * Bemutató kezdésekor a lista feltöltése, majd bemutatja az első lépést.<br>
   * Később pedig csak a következő lépés bemutatása.
   */
  private void stepForwardMemory() {
    String resultText;
    boolean cond1;
    parent.pGrid.setCellMark(parent.pGrid.getActCell(), C_RESULT, S_OFF);
    if (parent.lists.getStepCount() == 0) {
      baseRulesChain = setupRulesChain();
      setActRulesChain(getBaseRulesChain());
      parent.pGrid.showTable(OP_MEMORY);
      parent.setupFrame();
      createList();
      if (parent.lists.getStepCount() == 0) {
        parent.setToolbarButtonEnabled(B_BACK, S_OFF);
        parent.setToolbarButtonEnabled(B_FORWARD, S_ON);
        JOptionPane.showMessageDialog(SudokuPI.getpSudoku(), M_EMPTY_LISTS);
      } else {
        parent.lists.setStepOrder((short) 1);
        if (parent.pGrid.isTableResolved(OP_MEMORY)) {
          resultText = M_PUZZLE_RESOLVED_AND_STEPCOUNT + parent.lists.getStepCount();
          JOptionPane.showMessageDialog(SudokuPI.getpSudoku(), resultText);
        }
      }
    } else {
      resultsSetShow(W_FW, S_OFF, D_DO);
      assistsSetShow(S_OFF);
      if (parent.lists.getStepCount()+1 == parent.lists.getStepOrder()) {
        parent.setToolbarButtonEnabled(B_FORWARD, S_OFF);
        cond1 = parent.pGrid.isTableResolved(OP_SCREEN);
        resultText = cond1 ? M_PUZZLE_RESOLVED : M_PUZZLE_NOT_RESOLVED;
        JOptionPane.showMessageDialog(null, resultText);
      } else {
        parent.lists.incStepOrder();
      }
    }
    resultsSetShow(W_FW, S_ON, D_DONTDO);
    assistsSetShow(S_ON);
  }

  /**
   * Elkészíti a listát.
   */
  public void createList() {
    boolean resolved, correct, isMax;
    byte chainPos, iGetOut = 0, ruleGetOut = 0;
    byte rulesCount = parent.storeProp.activeRulesCount();
    boolean rulesGetOut = parent.storeProp.getRulesGetOutFromProp();
    do {
      chainPos = oneStep(OP_MEMORY, ruleGetOut);
      resolved = parent.pGrid.isTableResolved(OP_MEMORY);
      correct = parent.pGrid.isTableCorrect(OP_MEMORY);
      isMax = parent.lists.getStepCount() >= MAX_STEPS_IN_LISTS-1;
      if (chainPos > 0) {
        fixIt((short) (parent.lists.getStepCount()-1));
        if (parent.storeProp.getEncapsulateFromProp()) {
          changeChain(chainPos);
        }
      } else {
        if (rulesGetOut && !resolved) {
          if (rulesCount > iGetOut) {
            iGetOut++;
            ruleGetOut = getRuleOrder(iGetOut);
            if ((ruleGetOut == 1) && (rulesCount > iGetOut)) {
              iGetOut++;
            }
          }
          if (rulesCount > iGetOut) {
            ruleGetOut = getRuleOrder(iGetOut);
            parent.lists.clearAssistAndResultLists();
          }
        }
      }
    } while ((chainPos > 0) && !resolved && correct && !isMax);   
  }

  /**
   * A találat végrehajtását végzi el, hogy a következő lépés onnan induljon.
   * 
   * @param lastStep 
   */
  private void fixIt(short lastStep) {
    short posFrom = parent.lists.getResultPosition(lastStep);
    short posTo = parent.lists.getResultPosition((short) (lastStep + 1));
    for (short position = posFrom; position < posTo; position++) {
      DCoord coord = parent.lists.getResultCoord(position);
      byte value = parent.lists.getResultValue(position);
      DNumber number = new DNumber(new DWhere(OP_MEMORY, coord), value);
      if (parent.lists.isResultTypeResult(position) == T_RESULT) {
        parent.pGrid.setGridResult(number);
      } else {
        DState state = new DState(number, S_OFF);
        parent.pGrid.setCandidateState(state);
      }
    }
  }

  /**
   * Az algoritmusokban haladás, míg nincs találat, vagy a keresési lánc végére
   * érve a keresés befejezése.
   *
   * @param where hol (OP_SCREEN - OP_MEMORY)
   * @param ruleStepOver
   * @return a talált szabály sorszáma
   */
  public byte oneStep(byte where, byte ruleStepOver) {
    boolean isResult = false;
    byte chainIndex = 1;
    byte appRule = 0;
    boolean cond1, cond2, cond3;
    do {
      isResult = false;
      appRule = getRuleOrder(chainIndex);
      cond2 = (parent.storeProp.getRulesGetOutFromProp() == S_ON);
      cond3 = cond2 && (appRule == ruleStepOver);
      byte category = CATEGORY[(cond3 ? 0 : appRule)];
      switch (category) {
        case 1:
          isResult = parent.rules.nakedSingle(where);
          break;
        case 2:
          isResult = parent.rules.hidden(where, appRule);
          break;
        case 3:
          isResult = parent.rules.interSection(where);
          break;
        case 4:
          isResult = parent.rules.naked(where, appRule);
          break;
        case 5:
          isResult = parent.rules.xWings(where);
          break;
      } // switch
      if (!isResult) {
        chainIndex++;
      } else {
        parent.lists.incStepCount();
        setStepPosition();
      }
      cond1 = (chainIndex <= parent.storeProp.activeRulesCount());
    } while (cond1 && (!isResult) && (!parent.pGrid.isTableResolved(where)));
    return (isResult ? appRule : 0);
  } // oneStep

  /**
   * Beállítja az aktuális lépés pozícióját a lépéskezdeteket tartalmazó tömbben..
   */
  private void setStepPosition() {
    short sc = parent.lists.getStepCount();
    parent.lists.setAssistPosition(sc, parent.lists.getAssistSize());
    parent.lists.setResultPosition(sc, parent.lists.getResultSize());
  }

  /**
   * Az adott cellához tatozó sor-oszlop-blokk celláiban megkeresi az adott
   * jelöltértéket, ha talál, akkor a listához adja. <br><br>
   * Ezt nevezi egy pár szakirodalom nulladik szabálynak.
   * 
   * (csak a nakedSingle használja)
   *
   * @param stepOrder aktuális lépés
   * @param number
   */
  public void rule_Cleaning(short stepOrder, DNumber number) {
    byte where = number.getWhere();
    byte result = number.getNumber();
    DCoord coordContact;
    byte row, col;
    for (byte offset = 0; offset < 9; offset++) {
      coordContact = new DCoord(number.getCoord().getRow(), offset);
      if (!coordContact.equals(number.getCoord())) {
        DNumber num = new DNumber(new DWhere(where, coordContact), result);
        turnOffCandidate(stepOrder, num);
      }
      coordContact = new DCoord(offset, number.getCoord().getCol());
      if (!coordContact.equals(number.getCoord())) {
        DNumber num = new DNumber(new DWhere(where, coordContact), result);
        turnOffCandidate(stepOrder, num);
      }
      row = (byte) ((number.getRow() - (number.getCoord().getRow() % 3)) + (offset / 3));
      col = (byte) ((number.getCol() - (number.getCoord().getCol() % 3)) + (offset % 3));
      coordContact = new DCoord(row, col);
      if (!coordContact.equals(number.getCoord())) {
        DNumber num = new DNumber(new DWhere(where, coordContact), result);
        turnOffCandidate(stepOrder, num);
      }
    }
  }

  /**
   * Az adott jelölt-érték törlése az adott cellához kapcsolódó munkaterületekről.
   * 
   * @param number
   * @param stepOrder
   */
  public void turnOffCandidate(short stepOrder, DNumber number) {
    if (parent.pGrid.isCandidateStateOn(number)) {
      if (number.getWhere() == OP_MEMORY) {
        parent.lists.addResult(stepOrder, (byte) 0, number, T_POSSIBILITY, "");
      }
      byte where = number.getWhere();
      byte value = number.getNumber();
      DNumber num = new DNumber(new DWhere(where, number.getCoord()), value);
      parent.pGrid.setCandidateState(new DState(num, S_OFF));
    }
  }

  /**
   * Azt adja vissza, hogy tart e a bemutató.
   * 
   * @return 
   */
  public boolean isShowStarted() {
    return showStarted;
  }

  /**
   * A bemutató folyamatának létét állítja be.
   * 
   * @param state
   */
  public void setShowStarted(boolean state) {
    showStarted = state;
  }
  
  /**
   * Visszaadja az index érték szerinti szabály sorszámát.
   * 
   * @param index
   * @return 
   */
  public byte getRuleOrder(byte index) {
    return Byte.parseByte(baseRulesChain.substring((index-1)*2, ((index-1)*2)+2));
  }

  /**
   * Összeállítja a lépések listáját, majd átadja a baseRulesChain változónak.
   * 
   * @return 
   */
  public String setupRulesChain() {
    String chain = "";
    if (parent.storeProp.getSearchWayFromProp() == 1) {   // előre
      if (parent.storeProp.getNakedSinglePosFromProp() == 1) {
        chain ="01";
        for (byte c = 2; c < RULE_COUNT+1; c++) {
          if (parent.storeProp.getRuleStateFromProp(c) == S_ON) {
            chain += Basics.padR("" + c, "0", 2);
          }
        }
      } else {
        for (byte c = 1; c < RULE_COUNT+1; c++) {
          if (parent.storeProp.getRuleStateFromProp(c) == S_ON) {
            chain += Basics.padR("" + c, "0", 2);
          }
        }
      }
    } else {                                              // vissza
      if (parent.storeProp.getNakedSinglePosFromProp() == 1) {
        chain = "01";
        for (byte c = RULE_COUNT; c > 1; c--) {
          if (parent.storeProp.getRuleStateFromProp(c) == S_ON) {
            chain += Basics.padR("" + c, "0", 2);
          }
        }
      } else {
        for (byte c = RULE_COUNT; c > 0; c--) {
          if (parent.storeProp.getRuleStateFromProp(c) == S_ON) {
            chain += Basics.padR("" + c, "0", 2);
          }
        }
      }
    }
    return chain;
  }

  /**
   * Visszaadja az alap szabályláncot.
   * 
   * @return 
   */
  public String getBaseRulesChain() {
    return baseRulesChain;
  }
  
  /**
   * Beállítja az alap szabályláncot.
   * 
   * @param chain 
   */
  public void setBaseRulesChain(String chain) {
    baseRulesChain = chain;
  }

  /**
   * Visszaadja az aktuális szabályláncot.
   * 
   * @return 
   */
  public String getActRulesChain() {
    return actRulesChain;
  }

  /**
   * Beállítja az aktuális szabályláncot.
   * 
   * @param chain 
   */
  public void setActRulesChain(String chain) {
    actRulesChain = chain;
  }

  /**
   * Az adott pozíció előtti részt áthelyezi a string végére.
   * Ez a szabály-betokozás esetén használandó. A számozás 1-el kezdődik.
   * Ha a szabály-betokozás aktív, az azt jelenti, hogy annál a szabálynál, ahol
   * találat született, az előre kerül. Ennek függvénye az is, hogy a Naked Single
   * szabály fixen az első e, mert ilyenkor, csak az utánna lévő pozíciók változnak.
   * 
   * @param order 
   */
  public void changeChain(short order) {
    String first = (parent.storeProp.getNakedSinglePosFromProp() == 1  ? "01" : "");
    if (order >= (parent.storeProp.getNakedSinglePosFromProp() == 1 ? 3 : 2)) {
      String second = getActRulesChain().substring((order - 1) * 2);
      String third = getActRulesChain().substring(first.length(), (order - 1) * 2);
      setActRulesChain(first + second + third);
    }
  }

}
