/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.Lists;

import SudokuPIPkg.DataForms.DCoord;

/**
 * Ez az osztály ahhoz kell, hogy az AssistList &lt;ArrayList&gt; kezelhesse
 * a segédcellák listáját, <br>valamint ebből öröklődik az LResult osztály.
 */
public class LAssist {

  /**
   * A lépés sorszámának tárolója
   */
  private short stepOrder;

  /**
   * A rácson belüli pozíció tárolója
   */
  private DCoord coord;

  /**
   * A találat típusa
   * true - A keresési eredmény cellája vagy eredmény-érték
   * false - A további munkaterület cellája vagy lehetőség-érték
   */
  private boolean type;

  /**
   * Konstruktor kapott értékekkel
   *
   * @param stepOrder lépés sorszáma
   * @param coord rácson belüli pozíció
   * @param type típus jelölés
   */
  public LAssist(short stepOrder, DCoord coord, boolean type) {
    this.stepOrder = stepOrder;
    this.coord = coord;
    this.type = type;
  }

  /**
   * Konstruktor 0-ás lépés-sorszámmal
   *
   * @param coord rácson belüli pozíció
   * @param type típus jelölés
   */
  public LAssist(DCoord coord, boolean type) {
    this((short) 0, coord, type);
  }

  /**
   * Konstruktor alapértékekkel
   */
  public LAssist() {
    this((short) 0, new DCoord((byte) 0, (byte) 0), false);
  }

  /**
   * A cella rácson belüli pozíciójának oszlop értékét adja vissza
   *
   * @return oszlop
   */
  public DCoord getCoord() {
    return coord;
  }

  /**
   * A lépés sorszámot adja vissza
   *
   * @return lépéssorszám
   */
  public short getStepOrder() {
    return stepOrder;
  }

  public boolean isTypeResult() {
    return type;
  }

  /**
   * Oszlop beállítása
   *
   * @param coord
   */
  public void setCoord(DCoord coord) {
    this.coord = coord;
  }

  /**
   * Lépéssorszám beállítása
   *
   * @param stepOrder lépéssorszám
   */
  public void setStepOrder(short stepOrder) {
    this.stepOrder = stepOrder;
  }

  /**
   * Megoldás-típus beállítása
   * 
   * @param resultType megoldás érték vagy cella típusa
   */
  public void setType(boolean resultType) {
    this.type = resultType;
  }

  /**
   * Alapértékek beállítása az átadott paraméterek alapján
   *
   * @param stepOrder lépés sorszáma
   * @param coord rácson belüli pozíció
   * @param resultType típus jelölés
   */
  public void setValues(short stepOrder, DCoord coord, boolean resultType) {
    this.stepOrder = stepOrder;
    this.coord = coord;
    this.type = resultType;
  }

  @Override
  public String toString() {
    return stepOrder + ".  " + getCoord().toString() + " - " +
            (type ? "Megoldás helye" : "Munkaterület egyéb cellája");
  }

}
