/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.Lists;

import SudokuPIPkg.Basis.IConst;
import SudokuPIPkg.DataForms.DWhere;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.DataForms.DNumber;

public class LResult extends LAssist {

  /**
   * A találati függvény neve.
   */
  private byte ruleOrder;

  /**
   * Eredmény, illetve jelölt érték, mely a resType értékétől függ.
   */
  private byte value;

  /**
   * Megjegyzés.
   */
  private String comment;

  /**
   * Az eredményeket tároló konstruktor
   *
   * @param stepOrder lépés sorszáma
   * @param ruleOrder szabály sorszáma
   * @param value
   * @param valType eredmény tipusa,
   *        azaz RESULT=eredmény-érték vagy POSSIBILITY=lehetőség-érték
   * @param comment
   */
  public LResult(short stepOrder, byte ruleOrder, DNumber value, boolean valType, String comment) {
    setStepOrder(stepOrder);
    this.ruleOrder = ruleOrder;
    this.setCoord(value.getCoord());
    this.value = value.getNumber();
    setType(valType);
    this.comment = comment;
  }

  /**
   * Az eredményeket tároló konstruktor
   *
   * @param ruleOrder szabály sorszáma
   * @param coord
   * @param value eredmény String
   * @param valType eredmény tipusa,
   * azaz RESULT=eredmény-érték vagy POSSIBILITY=lehetőség-érték
   * @param comment
   */
  public LResult(byte ruleOrder, DCoord coord, String value, boolean valType, String comment) {
    this((short) 0, ruleOrder, new DNumber(new DWhere((byte) 0, coord), Byte.parseByte(value)), valType, comment);
  }

  /**
   * Konstruktor null értékekkel.
   */
  public LResult() {
    this((short) 0, (byte) 0, new DNumber(new DWhere((byte) 0, new DCoord((byte) 0, (byte) 0)), (byte) 0), true, "");
  }

  /**
   * A szabály sorszámát adja vissza.
   *
   * @return
   */
  public byte getRuleOrder() {
    return ruleOrder;
  }

  /**
   * A talált értéket adja vissza.
   *
   * @return
   */
  public byte getValue() {
    return value;
  }
  
  /**
   * Megjegyzés szövegét adja vissza.
   *
   * @return
   */
  public String getComment() {
    return comment;
  }

  /**
   * A megjelenítendő megjegyzés szövegét adja vissza
   * 
   * @return 
   */
  public String showComment() {
    return (getComment().isEmpty() ? "" : " [" + getComment() + "]");
  }

  @Override
  public String toString() {
    return getStepOrder() + ". lépés:" + getCoord().toString() + ","
            + IConst.RULESSHOW[ruleOrder] + "  érték : " + value + ", Típus: "
            + (isTypeResult() ? "Megoldás" : "Lehetőség") + " " + showComment();
  }
}
